# encoding=UTF-8
from django.core.files.base import File
import json
import os
from django.conf import settings
from django.test import TestCase
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.db import IntegrityError
from elections.forms.election_form import AnswerForm
from django.template import Template, Context
from django.utils import simplejson as json
from django.utils.translation import ugettext as _

from elections.models import Election, Candidate, Category, PersonalData, \
                             BackgroundCategory, Background, PersonalDataCandidate
from elections.forms import ElectionForm, ElectionUpdateForm, PersonalDataForm, \
                            BackgroundCategoryForm, BackgroundForm, QuestionForm, \
                            CategoryForm, ElectionLogoUpdateForm, ElectionStyleUpdateForm
from elections.views import ElectionRedirectView

import random
import string

dirname = os.path.dirname(os.path.abspath(__file__))

class ElectionTagsTests(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='doe',
                                                password='joe',
                                                email='doe@joe.cl')
        self.not_user = User.objects.create_user(username='joe',
                                                password='joe',
                                                email='doe@joe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz')
        self.election2, created = Election.objects.get_or_create(name='BarBaz2',
                                                            owner=self.user,
                                                            slug='barbaz2')
        self.candidate = Candidate.objects.create(name='Bar Baz',
                                                            election=self.election)
        self.candidate2, created = Candidate.objects.get_or_create(
                                                            name='Bar Baz',
                                                            election=self.election2)
    
    def test_create_link_for_updating_election_data(self):
        template = Template('{% load election_tags %}{% link_to_updating_this_election user election %}')
        
        
        context = Context({"user": self.user, "election": self.election})
        election_update_url = reverse('election_update',kwargs={'slug':self.election.slug})
        expected_html = u'<span class="goedit"><a href="'+election_update_url+u'">Editar Elección</a></span>'
        
        self.assertEqual(template.render(context), expected_html)     
    
    def test_if_is_not_the_owner(self):
        template = Template('{% load election_tags %}{% link_to_updating_this_election user election %}')
        
        
        context = Context({"user": self.not_user, "election": self.election})
        expected_html = u''
        self.assertEqual(template.render(context), expected_html)
        
        
    def test_if_there_is_no_logged_user(self):
        template = Template('{% load election_tags %}{% link_to_updating_this_election user election %}')
        
        
        context = Context({"user": None, "election": self.election})
        expected_html = u''
        self.assertEqual(template.render(context), expected_html)

class ElectionModelTest(TestCase):
    def test_create_election(self):
        user, created = User.objects.get_or_create(username='joe')
        election, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=user,
                                                           slug='barbaz',
                                                           description='esta es una descripcion',
                                                           date='27 de Diciembre')
        self.assertTrue(created)
        self.assertEqual(election.name, 'BarBaz')
        self.assertEqual(election.owner, user)
        self.assertEqual(election.slug, 'barbaz')
        self.assertEqual(election.date, '27 de Diciembre')
        self.assertEqual(election.description, 'esta es una descripcion')
        self.assertEqual(election.published,False)
        self.assertEqual(election.custom_style, '')
        self.assertEqual(election.highlighted, False)

    def test_create_election_with_dependent_displaying_personal_data(self):
        user, created = User.objects.get_or_create(username='joe')
        election, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=user,
                                                           slug='barbaz',
                                                           description='esta es una descripcion',
                                                           date='27 de Diciembre')


        self.assertTrue(election.should_display_empty_personal_data)

    def test_create_election_without_slug(self):
        user, created = User.objects.get_or_create(username='joe')
        election, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=user,
                                                           description='esta es una descripcion',
                                                           date='27 de Diciembre')

        self.assertTrue(created)
        self.assertEqual(election.slug,'barbaz')


    def test_create_two_elections_same_name_without_slug(self):
        user, created = User.objects.get_or_create(username='joe')
        election1, created1 = Election.objects.get_or_create(name='BarBaz',
                                                           owner=user,
                                                           description='esta es una descripcion1',
                                                           date='27 de Diciembre')

        election2, created2 = Election.objects.get_or_create(name='BarBaz',
                                                           owner=user,
                                                           description='esta es una descripcion2',
                                                           date='27 de Diciembre')

        self.assertTrue(created2)
        self.assertEqual(election2.name, 'BarBaz')
        self.assertEqual(election2.slug,'barbaz2')

        election3, created3 = Election.objects.get_or_create(name='BarBaz',
                                                           owner=user,
                                                           description='esta es una descripcion3',
                                                           date='27 de Diciembre')

        self.assertTrue(created3)
        self.assertEqual(election3.name, 'BarBaz')
        self.assertEqual(election3.slug,'barbaz3')

    def test_create_two_elections_same_name_without_slug_and_one_with_another(self):
        user, created = User.objects.get_or_create(username='joe')
        election1, created1 = Election.objects.get_or_create(name='BarBaz',
                                                           owner=user,
                                                           description='esta es una descripcion1',
                                                           date='27 de Diciembre')

        election2, created2 = Election.objects.get_or_create(name='BarBaz2',
                                                           owner=user,
                                                           description='esta es una descripcion2',
                                                           date='27 de Diciembre')

        self.assertTrue(created2)
        self.assertEqual(election2.name, 'BarBaz2')
        self.assertEqual(election2.slug,'barbaz2')

        election3, created3 = Election.objects.get_or_create(name='BarBaz',
                                                           owner=user,
                                                           description='esta es una descripcion3',
                                                           date='27 de Diciembre')

        self.assertTrue(created3)
        self.assertEqual(election3.name, 'BarBaz')
        self.assertEqual(election3.slug,'barbaz3')




    def test_edit_embeded_style_for_election(self):
        user, created = User.objects.get_or_create(username='joe')
        election, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=user,
                                                           slug='barbaz',
                                                           description='esta es una descripcion',
                                                           date='27 de Diciembre')


        election.custom_style = "body {color:red;}"
        election.save()

        the_same_election= Election.objects.get(id=election.id)
        self.assertEqual(the_same_election.custom_style, election.custom_style)  

    def test_create_two_election_by_same_user_with_same_slug(self):
        user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        election = Election.objects.create(name='BarBaz',
                                                    owner=user,
                                                    slug='barbaz',
                                                    description='esta es una descripcion')

        self.assertRaises(IntegrityError, Election.objects.create,
                          name='FooBar', owner=user, slug='barbaz', description='whatever')

    def test_edit_election(self):
        user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        election, created = Election.objects.get_or_create(name='BarBaz',
                                                           owner=user,
                                                           slug = 'barbaz',
                                                           description='esta es una descripcion')
        election.name = 'Barba'
        election.save()
        election2 = Election.objects.get(slug='barbaz', owner=user)
        self.assertEquals(election.name, election2.name)

    def test_create_default_data(self):
        user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        election = Election.objects.create(name='Foo', owner=user, description='Lorem ipsum')
        datas = PersonalData.objects.filter(election=election)
        self.assertEqual(datas.count(), 4)
        self.assertEqual(datas.filter(label=u'Edad').count(), 1)
        self.assertEqual(datas.filter(label=u'Estado civil').count(), 1)
        self.assertEqual(datas.filter(label=u'Profesión').count(), 1)
        self.assertEqual(datas.filter(label=u'Género').count(), 1)

    def test_create_default_background_categories(self):
        user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        election = Election.objects.create(name='Foo', owner=user, description='Lorem ipsum')
        categories = BackgroundCategory.objects.filter(election=election)
        self.assertEqual(categories.count(), 2)
        self.assertEqual(categories.filter(name=u'Educación').count(), 1)
        self.assertEqual(categories.filter(name=u'Antecedentes laborales').count(), 1)
        category = categories.get(name=u'Educación')
        backgrounds = Background.objects.filter(category=category)
        self.assertEqual(backgrounds.count(), 3)
        self.assertEqual(backgrounds.filter(name=u'Educación primaria').count(), 1)
        self.assertEqual(backgrounds.filter(name=u'Educación secundaria').count(), 1)
        self.assertEqual(backgrounds.filter(name=u'Educación superior').count(), 1)
        category = categories.get(name=u'Antecedentes laborales')
        backgrounds = Background.objects.filter(category=category)
        self.assertEqual(backgrounds.filter(name=u'Último trabajo').count(), 1)
        

    def test_create_default_categories_questions_and_answers(self):
        user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        election = Election.objects.create(name='Foo', owner=user, description='Lorem ipsum')
        question_categories = Category.objects.filter(election=election)
        self.assertEquals(question_categories.count(),1)
        self.assertEquals(question_categories[0].name,u'Educación')
        questions = question_categories[0].question_set.all()
        self.assertEquals(questions.count(),2)
        self.assertEquals(questions[0].question,u'¿Crees que Chile debe tener una educación gratuita?')

        first_question = questions[0]
        self.assertEquals(first_question.answer_set.count(),2)
        self.assertEquals(first_question.answer_set.all()[0].caption,u"Sí")
        self.assertEquals(first_question.answer_set.all()[1].caption,u"No")
        self.assertEquals(questions[1].question,u'¿Estas de acuerdo con la desmunicipalización?')

        second_question = questions[1]
        self.assertEquals(second_question.answer_set.count(),2)
        self.assertEquals(second_question.answer_set.all()[0].caption,u"Sí")
        self.assertEquals(second_question.answer_set.all()[1].caption,u"No")

class ElectionPhotoUpdateViewFormTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password=PASSWORD, email='joe@exmaple.net')
        self.user2 = User.objects.create_user(username='pepito', password=PASSWORD, email='pepito@exmaple.net')
        self.election = Election.objects.create(name='election', slug='election', owner=self.user)
        self.url = reverse('update_election_photo', kwargs={'pk': self.election.pk})
        self.new_file = open(os.path.join(dirname, 'media/dummy_logo.jpg'), 'rb')
    
    def test_get_form_as_owner(self):
        self.client.login(username=self.user.username, password=PASSWORD)
        response = self.client.get(self.url)
        self.assertEquals(response.status_code, 200)
        self.assertTemplateUsed(response, 'elections/updating/election_logo_form.html')
        self.assertTrue('form' in response.context)
        self.assertIsInstance(response.context['form'], ElectionLogoUpdateForm)
        self.assertTrue('election' in response.context)
        self.assertEqual(response.context['election'], self.election)

    def test_post_new_image_as_owner(self):
        self.client.login(username=self.user.username, password=PASSWORD)
        data = {
            'logo': self.new_file
        }
        response = self.client.post(self.url, data)
        self.new_file.seek(0)
        self.assertRedirects(response, reverse('election_update', 
                                       kwargs={'slug': self.election.slug}))
        election = Election.objects.get(pk=self.election.pk)
        self.assertEquals(election.logo.file.read(), self.new_file.read())
        os.unlink(election.logo.path)

        
    def test_get_form_as_no_user(self):
        response = self.client.get(self.url)
        self.assertRedirects(response, '/accounts/login/?next=/election/'+str(self.election.pk)+'/update_election_photo')
        
    def test_get_form_as_user_but_no_owner(self):
        self.client.login(username=self.user2.username, password=PASSWORD)
        response = self.client.get(self.url)
        self.assertEquals(response.status_code, 404)   



class ElectionCustomStyleUpdateView(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password=PASSWORD, email='joe@exmaple.net')
        self.user2 = User.objects.create_user(username='doe', password=PASSWORD, email='doe@exmaple.net')
        self.election = Election.objects.create(name='election', slug='election', owner=self.user)
        self.url = reverse('update_custom_style', kwargs={'slug': self.election.slug})

    def test_get_form_for_updating_style(self):
        self.client.login(username=self.user.username, password=PASSWORD)
        response = self.client.get(self.url)
        self.assertEquals(response.status_code, 200)
        self.assertTemplateUsed(response, 'elections/updating/election_style_updating.html')
        self.assertTrue('form' in response.context)
        self.assertIsInstance(response.context['form'], ElectionStyleUpdateForm)
        self.assertTrue('election' in response.context)
        self.assertEqual(response.context['election'], self.election)

    def test_post_new_image_as_owner(self):
        self.client.login(username=self.user.username, password=PASSWORD)
        data = {
            'custom_style': 'body {background-color:red;}'
        }
        response = self.client.post(self.url, data)

        self.assertRedirects(response, reverse('update_custom_style', 
                                       kwargs={'slug': self.election.slug}))
        election = Election.objects.get(slug=self.election.slug)
        self.assertEquals(election.custom_style, data['custom_style'])


    def test_get_form_as_no_user(self):
        response = self.client.get(self.url)
        self.assertRedirects(response, '/accounts/login/?next=/election/'+str(self.election.slug)+'/update_style')
        
    def test_get_form_as_user_but_no_owner(self):
        self.client.login(username=self.user2.username, password=PASSWORD)
        response = self.client.get(self.url)
        self.assertEquals(response.status_code, 404)

    def test_get_election_created_with_the_same_slug_but_different_users(self):
        fiera = User.objects.create_user(username='Fiera', password='Feroz', email='joe@doe.cl')
        election = Election.objects.create(name='elec foo', slug='foobarbar', owner=fiera)
        user2 = User.objects.create_user(username='Doe', password='doe', email='joe@doe.cl')
        election2 = Election.objects.create(name='foobar', slug='foobarbar', owner=user2)

        self.client.login(username=fiera.username, password='Feroz')
        url = reverse('update_custom_style', kwargs={'slug': election.slug})
        response = self.client.get(reverse('election_update',
                                    kwargs={'slug': election.slug}))

        self.assertEquals(response.status_code, 200) 

    def test_post_election_created_with_the_same_slug_but_different_users(self):
        fiera = User.objects.create_user(username='Fiera', password='Feroz', email='joe@doe.cl')
        election = Election.objects.create(name='elec foo', slug='foobarbar', owner=fiera)
        user2 = User.objects.create_user(username='Doe', password='doe', email='joe@doe.cl')
        election2 = Election.objects.create(name='foobar', slug='foobarbar', owner=user2)

        self.client.login(username=fiera.username, password='Feroz')
        url = reverse('update_custom_style', kwargs={'slug': election.slug})
        data = {
            'custom_style': 'body {background-color:red;}'
        }
        response = self.client.post(url, data)

        self.assertEquals(response.status_code, 302)


class ElectionDetailViewTest(TestCase):
    def test_detail_existing_election_view(self):
        user = User.objects.create(username='foobar')
        election = Election.objects.create(name='elec foo', slug='elec-foo', owner=user, published=True)
        url = reverse('election_detail',
            kwargs={
                'username': user.username,
                'slug': election.slug
            })
        response = self.client.get(url)
        self.assertTemplateUsed(response, "elections/base.html")
        self.assertTemplateUsed(response, "elections/election_detail_profiles.html")
        self.assertEquals(response.status_code, 200)
        self.assertTrue('election' in response.context)
        self.assertEquals(response.context['election'], election)

    def test_detail_non_existing_election_view(self):
        user = User.objects.create(username='foobar')
        response = self.client.get(reverse('election_detail',
                                           kwargs={
                                               'username': user.username,
                                               'slug': 'asd-asd'}))
        self.assertEquals(response.status_code, 404)

    def test_detail_non_existinmeg_election_for_user_view(self):
        user = User.objects.create(username='foobar')
        user2 = User.objects.create(username='barbaz')
        election = Election.objects.create(name='elec foo', slug='elec-foo', owner=user2, published=True)
        response = self.client.get(reverse('election_detail',
                                           kwargs={
                                               'username': user.username,
                                               'slug': election.slug}))
        self.assertEquals(response.status_code, 404)
    def test_detail_non_published_election_for_user_view(self):
        user = User.objects.create(username='foobar')
        election = Election.objects.create(name='elec foo', slug='elec-foo', owner=user)
        response = self.client.get(reverse('election_detail',
                                           kwargs={
                                               'username': user.username,
                                               'slug': election.slug}))
        self.assertEquals(response.status_code, 404)

class ElectionCompareViewTest(TestCase):
    def test_compare_existing_election_view(self):
        user = User.objects.create(username='foobar')
        election = Election.objects.create(name='elec foo', slug='elec-foo', owner=user, published=True)
        response = self.client.get(reverse('election_compare',
                                           kwargs={
                                               'username': user.username,
                                               'slug': election.slug}))
        self.assertEquals(response.status_code, 200)
        self.assertTrue('election' in response.context)
        self.assertEquals(response.context['election'], election)

    def test_compare_non_existing_election_view(self):
        user = User.objects.create(username='foobar')
        response = self.client.get(reverse('election_compare',
                                           kwargs={
                                               'username': user.username,
                                               'slug': 'asd-asd'}))
        self.assertEquals(response.status_code, 404)

    def test_compare_non_existing_election_for_user_view(self):
        user = User.objects.create(username='foobar')
        user2 = User.objects.create(username='barbaz')
        election = Election.objects.create(name='elec foo', slug='elec-foo', owner=user2)
        response = self.client.get(reverse('election_compare',
                                           kwargs={
                                               'username': user.username,
                                               'slug': election.slug}))
        self.assertEquals(response.status_code, 404)

    def test_compare_an_election_with_an_existing_slug(self):
        user = User.objects.create(username='foobar')
        election = Election.objects.create(name='elec foo', slug='elec-foo', owner=user, published=True)
        user2 = User.objects.create(username='barbaz')
        election2 = Election.objects.create(name='elec foo', slug='elec-foo', owner=user2, published=True)
        response = self.client.get(reverse('election_compare',
                                           kwargs={
                                               'username': user.username,
                                               'slug': election.slug}))

        self.assertEquals(response.status_code, 200)


    def test_compare_one_candidate_view(self):
        user = User.objects.create(username='foobar')
        election = Election.objects.create(name='elec foo', slug='elec-foo', owner=user, published=True)
        f = open(os.path.join(dirname, 'media/dummy.jpg'), 'rb')
        candidate = Candidate.objects.create(name='bar baz', election=election, photo=File(f))
        response = self.client.get(reverse('election_compare_one_candidate',
                                           kwargs={
                                               'username': user.username,
                                               'slug': election.slug,
                                               'first_candidate_slug': candidate.slug}))
        self.assertEquals(response.status_code, 200)

    def test_compare_one_candidate_mismatch_view(self):
        user = User.objects.create(username='foobar')
        election = Election.objects.create(name='elec foo', slug='elec-foo', owner=user)
        candidate = Candidate.objects.create(name='bar baz', election=election)
        response = self.client.get(reverse('election_compare_one_candidate',
                                           kwargs={
                                               'username': user.username,
                                               'slug': election.slug,
                                               'first_candidate_slug': 'asdf'}))
        self.assertEquals(response.status_code, 404)

    def test_compare_two_candidates_view(self):
        user = User.objects.create(username='foobar')
        election = Election.objects.create(name='elec foo', slug='elec-foo', owner=user, published=True)
        f = open(os.path.join(dirname, 'media/dummy.jpg'), 'rb')
        first_candidate = Candidate.objects.create(name='bar baz', election=election, photo=File(f))
        second_candidate = Candidate.objects.create(name='tar taz', election=election, photo=File(f))
        category = Category.objects.create(name='asdf', election=election, slug='asdf')
        response = self.client.get(reverse('election_compare_two_candidates',
                                           kwargs={
                                               'username': user.username,
                                               'slug': election.slug,
                                               'first_candidate_slug': first_candidate.slug,
                                               'second_candidate_slug': second_candidate.slug,
                                               'category_slug': category}))
        self.assertEquals(response.status_code, 200)

    def test_compare_one_candidate_two_times(self):
        user = User.objects.create(username='foobar')
        election = Election.objects.create(name='elec foo', slug='elec-foo', owner=user, published=True)
        f = open(os.path.join(dirname, 'media/dummy.jpg'), 'rb')
        first_candidate = Candidate.objects.create(name='bar baz', election=election, photo=File(f))
        second_candidate = first_candidate
        category = Category.objects.create(name='asdf', election=election, slug='asdf')
        response = self.client.get(reverse('election_compare_two_candidates',
                                           kwargs={
                                               'username': user.username,
                                               'slug': election.slug,
                                               'first_candidate_slug': first_candidate.slug,
                                               'second_candidate_slug': second_candidate.slug,
                                               'category_slug': category}))
        self.assertEquals(response.status_code, 404)


    def test_compare_two_candidates_category_mismatch_view(self):
        user = User.objects.create(username='foobar')
        election = Election.objects.create(name='elec foo', slug='elec-foo', owner=user, published=True)
        first_candidate = Candidate.objects.create(name='bar baz', election=election)
        second_candidate = Candidate.objects.create(name='tar taz', election=election)
        category = Category.objects.create(name='asdf', election=election, slug='asdf')
        response = self.client.get(reverse('election_compare_two_candidates',
                                           kwargs={
                                               'username': user.username,
                                               'slug': election.slug,
                                               'first_candidate_slug': first_candidate.slug,
                                               'second_candidate_slug': second_candidate.slug,
                                               'category_slug': 'asdf2'}))
        self.assertEquals(response.status_code, 404)

    def test_compare_two_candidates_first_candidate_mismatch_view(self):
        user = User.objects.create(username='foobar')
        election = Election.objects.create(name='elec foo', slug='elec-foo', owner=user, published=True)
        first_candidate = Candidate.objects.create(name='bar baz', election=election)
        second_candidate = Candidate.objects.create(name='tar taz', election=election)
        category = Category.objects.create(name='asdf', election=election, slug='asdf')
        response = self.client.get(reverse('election_compare_two_candidates',
                                           kwargs={
                                               'username': user.username,
                                               'slug': election.slug,
                                               'first_candidate_slug': 'asdf',
                                               'second_candidate_slug': second_candidate.slug,
                                               'category_slug': category}))
        self.assertEquals(response.status_code, 404)

    def test_compare_two_candidates_second_candidate_mismatch_view(self):
        user = User.objects.create(username='foobar')
        election = Election.objects.create(name='elec foo', slug='elec-foo', owner=user, published=True)
        first_candidate = Candidate.objects.create(name='bar baz', election=election)
        second_candidate = Candidate.objects.create(name='tar taz', election=election)
        category = Category.objects.create(name='asdf', election=election, slug='asdf')
        response = self.client.get(reverse('election_compare_two_candidates',
                                           kwargs={
                                               'username': user.username,
                                               'slug': election.slug,
                                               'first_candidate_slug': first_candidate.slug,
                                               'second_candidate_slug': 'asdf',
                                               'category_slug': category}))
        self.assertEquals(response.status_code, 404)

    def test_404_election_compare_asynchronous_call(self):
        user = User.objects.create(username='foobar')
        election = Election.objects.create(name='elec foo', slug='elec-foo', owner=user, published=True)
        first_candidate = Candidate.objects.create(name='bar baz', election=election)

        response = self.client.get(reverse('election_compare_asynchronous_call',
                                            kwargs={
                                                'username': user.username,
                                                'slug': election.slug,
                                                'candidate_slug': first_candidate.slug,
                                            }), HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        #405 means method not allowed
        self.assertEqual(response.status_code, 405)


    def test_comparison_with_only_one_candidate_is_being_selected(self):
        user = User.objects.create(username='foobar')
        election = Election.objects.create(name='elec foo', slug='elec-foo', owner=user, published=True)
        f = open(os.path.join(dirname, 'media/dummy.jpg'), 'rb')
        first_candidate = Candidate.objects.create(name='bar baz', election=election, photo=File(f))
        self.personal_data = PersonalData.objects.create(election=election, label='edad')
        self.personal_data_candidate = PersonalDataCandidate.objects.create(personal_data=self.personal_data,
            candidate=first_candidate,
            value=u'miles de años de edad')
        response = self.client.post(reverse('election_compare_asynchronous_call',
            kwargs={
                'username': user.username,
                'slug': election.slug,
                'candidate_slug': first_candidate.slug,
                }))

        self.assertEqual(response.status_code, 200)
        response_json = json.loads(response.content)
        expected_personal_data = {"edad":u"miles de años de edad"}
        self.assertTrue("edad" in response_json["personal_data"])
        self.assertEqual(expected_personal_data["edad"],response_json['personal_data']["edad"])


    def test_comparison_with_only_one_candidate_is_being_selected_and_the_candidate_does_not_have_photo(self):
        user = User.objects.create(username='foobar')
        election = Election.objects.create(name='elec foo', slug='elec-foo', owner=user, published=True)
        first_candidate = Candidate.objects.create(name='bar baz', election=election)


        response = self.client.post(reverse('election_compare_asynchronous_call',
            kwargs={
                'username': user.username,
                'slug': election.slug,
                'candidate_slug': first_candidate.slug,
                }))

        self.assertEqual(response.status_code, 200)

    def test_election_about(self):
        user = User.objects.create(username='foobar')
        election = Election.objects.create(name='elec foo', slug='elec-foo', owner=user)



    def test_comparison_of_two_candidates_with_no_category(self):
        user = User.objects.create(username='foobar')
        election = Election.objects.create(name='elec foo', slug='elec-foo', owner=user, published=True)
        first_candidate = Candidate.objects.create(name='bar baz', election=election)
        second_candidate = Candidate.objects.create(name='sec baz', election=election)
        url = reverse('election_compare_two_candidates_and_no_category',kwargs={
            'username': user.username,
            'slug':election.slug,
            'first_candidate_slug':first_candidate.slug,
            'second_candidate_slug':second_candidate.slug,
        })
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_comparison_of_the_same_candidate_with_no_category(self):
        user = User.objects.create(username='foobar')
        election = Election.objects.create(name='elec foo', slug='elec-foo', owner=user)
        first_candidate = Candidate.objects.create(name='bar baz', election=election)
        url = reverse('election_compare_two_candidates_and_no_category',kwargs={
            'username': user.username,
            'slug':election.slug,
            'first_candidate_slug':first_candidate.slug,
            'second_candidate_slug':first_candidate.slug,
            })
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

    def test_get_election_about(self):
        user = User.objects.create(username='foobar')
        election = Election.objects.create( name='elec foo', \
                                            slug='elec-foo', \
                                            owner=user,\
                                            description="This is a description of the election",\
                                            published=True
                                            )
        url = reverse('election_about',kwargs={
            'username': user.username,
            'slug': election.slug
        })
        response = self.client.get(url)
        self.assertContains(response,'election')
        self.assertEqual(response.context['election'], election)
        self.assertTemplateUsed(response, "elections/election_about.html")




class ElectionCreateViewTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')



    def test_create_election_by_user_without_login(self):
        response = self.client.get(reverse('election_create'))
        self.assertEquals(response.status_code, 302)

    def test_create_election_by_user_success(self):
        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('election_create'))

        self.assertTrue('form' in response.context)
        self.assertTrue(isinstance(response.context['form'], ElectionForm))

    def test_post_election_create_with_same_name(self):
        
        election = Election.objects.create(name='BarBaz', description='whatever', owner=self.user)

        self.client.login(username=self.user.username, password='doe')
        f = open(os.path.join(dirname, 'media/dummy.jpg'), 'rb')
        params = {'name': 'BarBaz', 'description': 'esta es una descripcion', 'logo': f,'information_source':'saque la info de un lugar'}
        response = self.client.post(reverse('election_create'), params, follow=True)
        f.close()
        first_election = Election.objects.get(owner=self.user, slug="barbaz")
        second_election = Election.objects.get(owner=self.user, slug="barbaz2")

        self.assertEquals(response.status_code, 200)
        self.assertEquals(Election.objects.filter(owner=self.user, name="BarBaz").count(), 2)
        self.assertEquals(first_election.name, election.name)
        self.assertEquals(second_election.name, params["name"])


        self.assertTemplateUsed(response,'elections/wizard/step_two.html')

    def test_invalid_creation_form(self):
        election_name = ''.join(random.choice(string.ascii_uppercase + string.digits) for x in range(300))
        #Haciendo que el nombre de la elección tenga más de 255 caractéres
        self.client.login(username=self.user.username, password='doe')
        f = open(os.path.join(dirname, 'media/dummy.jpg'), 'rb')
        params = {'name': election_name, 'description': 'esta es una descripcion', 'logo': f,'information_source':'saque la info de un lugar'}
        response = self.client.post(reverse('election_create'), params, follow=True)
        f.close()
        self.assertEquals(response.status_code, 200)
        self.assertTemplateUsed(response, 'elections/wizard/step_one.html')
        self.assertTrue(response.context['form'].errors.has_key('name'))
        


    def test_post_election_create_without_login(self):
        f = open(os.path.join(dirname, 'media/dummy.jpg'), 'rb')
        params = {'name': 'BarBaz', 'slug': 'barbaz', 'description': 'esta es una descripcion', 'logo': f}
        response = self.client.post(reverse('election_create'), params)
        f.close()

        self.assertEquals(response.status_code, 302)

    def test_post_election_create_logged(self):
        self.client.login(username='joe', password='doe')

        f = open(os.path.join(dirname, 'media/dummy.jpg'), 'rb')
        params = {'name': 'BarBaz', 'description': 'esta es una descripcion', 'logo': f,'information_source':'saque la info de un lugar'}
        response = self.client.post(reverse('election_create'), params, follow=True)
        f.seek(0)

        self.assertEquals(response.status_code, 200)
        qs = Election.objects.filter(name='BarBaz')
        self.assertEquals(qs.count(), 1)
        election = qs.get()
        self.assertEquals(election.name, 'BarBaz')
        self.assertEquals(election.slug, 'barbaz')
        self.assertEquals(election.description, 'esta es una descripcion')
        self.assertEquals(f.read(), election.logo.file.read())

        os.unlink(election.logo.path)
        self.assertEquals(election.owner, self.user)
        self.assertRedirects(response, reverse('candidate_create',
                                               kwargs={'election_slug': election.slug}))
        
    def test_template_step_one(self):
        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('election_create'))
        self.assertTemplateUsed(response,'elections/wizard/step_one.html')

class ElectionUpdateViewTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election = Election.objects.create(name='elec foo', slug='eleccion-la-florida', owner=self.user)

    def test_update_election_by_user_without_login(self):
        response = self.client.get(reverse('election_update', kwargs={'slug': self.election.slug}))
        self.assertEquals(response.status_code, 302)

    def test_update_election_by_user_success(self):
        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('election_update', kwargs={'slug': self.election.slug}))

        self.assertTrue('form' in response.context)
        self.assertTrue(isinstance(response.context['form'], ElectionUpdateForm))
        self.assertTrue('election' in response.context)
        self.assertEqual(response.context['election'], self.election)
    
    def test_template_election_basic_information_rendered(self):
        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('election_update', kwargs={'slug': self.election.slug}))
        
        self.assertTemplateUsed(response,'elections/updating/election_basic_information.html')

    def test_post_election_update_without_login(self):
        f = open(os.path.join(dirname, 'media/dummy.jpg'), 'rb')
        params = {'name': 'BarBaz', 'description': 'esta es una descripcion', 'logo': f}
        response = self.client.post(reverse('election_update', kwargs={'slug': self.election.slug}), params)
        f.close()

        self.assertEquals(response.status_code, 302)

    def test_get_election_update_strager_election(self):
        self.client.login(username='joe', password='doe')

        user2 = User.objects.create_user(username='Doe', password='doe', email='joe@doe.cl')
        election2 = Election.objects.create(name='foobar', slug='foobarbar', owner=user2)

        response = self.client.get(reverse('election_update',
                                    kwargs={'slug': election2.slug}))
        self.assertEqual(response.status_code, 404)

    def test_get_election_created_with_the_same_slug_but_different_users(self):
        fiera = User.objects.create_user(username='Fiera', password='Feroz', email='joe@doe.cl')
        election = Election.objects.create(name='elec foo', slug='foobarbar', owner=fiera)
        user2 = User.objects.create_user(username='Doe', password='doe', email='joe@doe.cl')
        election2 = Election.objects.create(name='foobar', slug='foobarbar', owner=user2)

        self.client.login(username=fiera.username, password='Feroz')
        response = self.client.get(reverse('election_update',
                                    kwargs={'slug': election.slug}))

        self.assertEquals(response.status_code, 200)


    def test_post_election_update_stranger_election(self):
        self.client.login(username='joe', password='doe')

        user2 = User.objects.create_user(username='Doe', password='doe', email='joe@doe.cl')
        election2 = Election.objects.create(name='foobar', slug='foobarbar', owner=user2)

        f = open(os.path.join(dirname, 'media/dummy.jpg'), 'rb')
        params = {'name': 'BarBaz', 'description': 'esta es una descripcion', 'logo': f}
        response = self.client.post(reverse('election_update',
                                        kwargs={'slug': election2.slug}),
                                    params)
        f.seek(0)
        self.assertEqual(response.status_code, 404)

    def test_post_election_update_logged(self):
        self.client.login(username='joe', password='doe')

        f = open(os.path.join(dirname, 'media/dummy.jpg'), 'rb')
        params = {'name': 'BarBaz', 'description': 'esta es una descripcion', 'logo': f,'information_source':u'me contó un pajarito'}
        response = self.client.post(reverse('election_update', kwargs={'slug': self.election.slug}), params, follow=True)
        f.seek(0)

        self.assertEquals(response.status_code, 200)
        qs = Election.objects.filter(name='BarBaz')
        self.assertEquals(qs.count(), 1)
        election = qs.get()
        self.assertEquals(election.name, 'BarBaz')
        self.assertEquals(election.slug, self.election.slug)
        self.assertEquals(election.description, 'esta es una descripcion')
        self.assertEquals(f.read(), election.logo.file.read())

        os.unlink(election.logo.path)
        self.assertEquals(election.owner, self.user)
        self.assertRedirects(response, reverse('election_update',
                                               kwargs={'slug': election.slug}))




    def test_post_election_update_does_not_update_published_status(self):
        published_election = Election.objects.create(name='elec foo', slug='la-terrible-de-eleccion', owner=self.user, published=True)
        self.client.login(username='joe', password='doe')
        f = open(os.path.join(dirname, 'media/dummy.jpg'), 'rb')
        params = {'name': 'BarBaz', 'description': 'esta es una descripcion', 'logo': f,'information_source':u'me contó un pajarito'}
        response = self.client.post(reverse('election_update', kwargs={'slug': published_election.slug}), params, follow=True)
        f.seek(0)
        election = Election.objects.get(id=published_election.id)

        self.assertTrue(election.published)


    def test_it_contains_the_election_full_url(self):
        username = 'joe'
        self.client.login(username=username, password='doe')
        response = self.client.get(reverse('election_update', kwargs={'slug': self.election.slug}))

        self.assertTrue('election_url' in response.context)
        url = response.context['election_url']
        self.assertTrue(url.startswith('http://'))
        self.assertTrue(url.endswith(reverse('election_detail', kwargs={'username':username, 'slug': self.election.slug})))


class ElectionUrlsTest(TestCase):
    def test_create_url(self):
        expected = '/election/create'
        result = reverse('election_create')
        self.assertEquals(result, expected)

    def test_pre_create_url(self):
        expected = '/election/pre_create'
        result = reverse('election_pre_create')
        self.assertEquals(result, expected)

    def test_detail_url(self):
        expected = '/juanito/eleccion-la-florida/'
        result = reverse('election_detail', kwargs={'username': 'juanito', 'slug': 'eleccion-la-florida'})
        self.assertEquals(result, expected)

    def test_compare_url(self):
        expected = '/juanito/eleccion-la-florida/compare'
        result = reverse('election_compare', kwargs={'username': 'juanito', 'slug': 'eleccion-la-florida'})
        self.assertEquals(result, expected)

    def test_compare_one_candidate_url(self):
        expected = '/juanito/eleccion-la-florida/compare/my-candidate'
        result = reverse('election_compare_one_candidate', kwargs={'username': 'juanito', 'slug': 'eleccion-la-florida', 'first_candidate_slug':'my-candidate'})
        self.assertEquals(result, expected)

    def test_compare_two_candidates_url(self):
        expected = '/juanito/eleccion-la-florida/compare/my-candidate/other-candidate/this-category'
        result = reverse('election_compare_two_candidates', kwargs={'username': 'juanito', 'slug': 'eleccion-la-florida', 'first_candidate_slug':'my-candidate', 'second_candidate_slug':'other-candidate', 'category_slug':'this-category'})
        self.assertEquals(result, expected)

    def test_update_url(self):
        expected = '/election/eleccion-la-florida/update'
        result = reverse('election_update', kwargs={'slug': 'eleccion-la-florida'})
        self.assertEquals(result, expected)

    def test_profiles_url(self):
        expected = '/juanito/eleccion-la-florida/profiles'
        result = reverse('election_detail_profiles', kwargs={'username': 'juanito', 'slug': 'eleccion-la-florida'})
        self.assertEquals(result, expected)

    def test_profiles_url(self):
        expected = '/juanito/eleccion-la-florida/gracias'
        result = reverse('election_detail_admin', kwargs={'username': 'juanito', 'slug': 'eleccion-la-florida'})
        self.assertEquals(result, expected)

        

class PrePersonalDataViewTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election = Election.objects.create(name='elec foo', slug='eleccion-la-florida', owner=self.user)

    def test_context(self):
        self.client.login(username='joe', password='doe')

        response = self.client.get(reverse('pre_personaldata', kwargs={'election_slug': self.election.slug}))
        self.assertEquals(response.status_code, 200)
        self.assertTrue('election' in response.context)
        self.assertEquals(response.context['election'], self.election)



class SharingYourElectionButton(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election = Election.objects.create(name='elec foo', slug='eleccion-la-florida', owner=self.user)

    def test_share_my_election(self):
        self.client.login(username='joe', password='doe')
        expected_url = '/election/eleccion-la-florida/share'
        url = reverse('share_my_election', kwargs={'slug': self.election.slug})
        self.assertEquals(url, expected_url)
        self.assertEqual(self.election.published,False)
        response = self.client.get(url)
        self.election=Election.objects.get(name='elec foo', slug='eleccion-la-florida', owner=self.user)
        self.assertEqual(self.election.published,True)
        self.assertTemplateUsed(response, 'elections/updating/share.html')
        self.assertEquals(response.context['election'], self.election)





PASSWORD = 'password'


class ElectionUpdateDataViewTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='Joe', password=PASSWORD, email='joe@doe.cl')
        self.election = Election.objects.create(name='Foo', owner=self.user, slug='foo')
        self.url = reverse('election_update_data', kwargs={'slug': self.election.slug})

    def test_get_not_logged(self):
        response = self.client.get(self.url)
        self.assertRedirects(response, settings.LOGIN_URL + '?next=' + self.url)

    def test_get_not_owned_election(self):
        stranger_user = User.objects.create_user(username='John', password=PASSWORD, email='john@doe.cl')
        self.client.login(username=stranger_user.username, password=PASSWORD)
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 404)

    def test_get_non_existing_election(self):
        url = reverse('election_update_data', kwargs={'slug': 'random_slug'})
        self.client.login(username=self.user.username, password=PASSWORD)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

    def test_get_owned_election(self):
        self.client.login(username=self.user.username, password=PASSWORD)
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTrue('election' in response.context)
        self.assertEqual(response.context['election'], self.election)
        self.assertTemplateUsed(response, 'elections/updating/questions.html')

        self.assertTrue('personaldata_form' in response.context)
        self.assertIsInstance(response.context['personaldata_form'], PersonalDataForm)

        self.assertTrue('backgroundcategory_form' in response.context)
        self.assertIsInstance(response.context['backgroundcategory_form'], BackgroundCategoryForm)

        self.assertTrue('background_form' in response.context)
        self.assertIsInstance(response.context['background_form'], BackgroundForm)

        self.assertTrue('question_form' in response.context)
        self.assertIsInstance(response.context['question_form'], QuestionForm)

        self.assertTrue('category_form' in response.context)
        self.assertIsInstance(response.context['category_form'], CategoryForm)

        self.assertTrue('answer_form' in response.context)
        self.assertIsInstance(response.context['answer_form'], AnswerForm)


PASSWORD = 'password'

class ElectionRedirectViewTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password=PASSWORD, email='joe@example.net')
        self.election = Election.objects.create(owner=self.user, name='Election', slug='election')
        self.candidate = Candidate.objects.create(election=self.election, name='Candidate')
        self.candidate2 = Candidate.objects.create(election=self.election, name='Candidate2')
        self.url = reverse('election_redirect')

    def test_non_existing_election(self):
        user = User.objects.create_user(username='doe', password=PASSWORD, email='doe@example.net')
        self.client.login(username=user.username, password=PASSWORD)
        response = self.client.get(self.url)
        self.assertRedirects(response, reverse('election_create'))

    def test_existing_one_election(self):
        self.client.login(username=self.user.username, password=PASSWORD)
        response = self.client.get(self.url)
        self.assertRedirects(response, reverse('candidate_data_update',
                                       kwargs={'election_slug': self.election.slug, 'slug': self.candidate.slug}))

    def test_existing_several_elections(self):
        election = Election.objects.create(name='Another Election', owner=self.user, slug='another-election')
        candidate = Candidate.objects.create(election=election, name='Candidate2')
        self.client.login(username=self.user.username, password=PASSWORD)
        response = self.client.get(self.url)
        self.assertRedirects(response, reverse('candidate_data_update',
                                       kwargs={'election_slug': election.slug, 'slug': candidate.slug}))

    def test_existing_election_without_candidates(self):
        election = Election.objects.create(name='Another Election', owner=self.user, slug='another-election', published=True)
        self.client.login(username=self.user.username, password=PASSWORD)
        response = self.client.get(self.url)
        self.assertRedirects(response, reverse('election_detail_admin',
                                       kwargs={'slug': election.slug, 'username': self.user.username}))

    def test_not_logged(self):
        response = self.client.get(self.url)
        self.assertRedirects(response, settings.LOGIN_URL + '?next=' + self.url)


class HomeTemplateView(TestCase):
    def test_it_brings_the_last_five_create_elections(self):
        self.user = User.objects.create_user(username='joe', password=PASSWORD, email='joe@example.net')
        election1 = Election.objects.create(owner=self.user, name='Election', slug='election1', published=True)
        election2 = Election.objects.create(owner=self.user, name='Election', slug='election2', published=True)
        election3 = Election.objects.create(owner=self.user, name='Election', slug='election3', published=True)
        election4 = Election.objects.create(owner=self.user, name='Election', slug='election4', published=True)
        election5 = Election.objects.create(owner=self.user, name='Election', slug='election5', published=True)
        election6 = Election.objects.create(owner=self.user, name='Election', slug='election6', published=True)
        self.url = reverse('home')
        response = self.client.get(self.url)
        self.assertTrue('last_elections' in response.context)
        elections = response.context['last_elections']
        self.assertTrue(elections.count() == 5)
        self.assertTrue(elections[0] == election6)

        
    def test_it_brings_the_last_created_and_published_elections(self):
        self.user = User.objects.create_user(username='joe', password=PASSWORD, email='joe@example.net')
        election1 = Election.objects.create(owner=self.user, name='Election', slug='election1', published=True)
        election2 = Election.objects.create(owner=self.user, name='Election', slug='election2', published=True)
        election3 = Election.objects.create(owner=self.user, name='Election', slug='election3')
        election4 = Election.objects.create(owner=self.user, name='Election', slug='election4')
        election5 = Election.objects.create(owner=self.user, name='Election', slug='election5')
        election6 = Election.objects.create(owner=self.user, name='Election', slug='election6')
        self.url = reverse('home')
        response = self.client.get(self.url)
        self.assertTrue('last_elections' in response.context)
        elections = response.context['last_elections']
        self.assertTrue(elections.count() == 2)
        self.assertTrue(elections[0] == election2)
        
    def test_it_brings_the_just_five_highlighted_elections(self):
        self.user = User.objects.create_user(username='joe', password=PASSWORD, email='joe@example.net')
        election1 = Election.objects.create(owner=self.user, name='Election', slug='election1', published=True, highlighted=True)
        election2 = Election.objects.create(owner=self.user, name='Election', slug='election2', published=True, highlighted=True)
        election3 = Election.objects.create(owner=self.user, name='Election', slug='election3', published=True, highlighted=True)
        election4 = Election.objects.create(owner=self.user, name='Election', slug='election4', published=True, highlighted=True)
        election5 = Election.objects.create(owner=self.user, name='Election', slug='election5', published=True, highlighted=True)
        election6 = Election.objects.create(owner=self.user, name='Election', slug='election6', published=True, highlighted=True)
        self.url = reverse('home')
        response = self.client.get(self.url)
        self.assertTrue('highlighted_elections' in response.context)
        elections = response.context['highlighted_elections']
        self.assertTrue(elections.count() == 5)

        
    # def test_it_brings_the_last_created_and_published_elections(self):
    #     self.user = User.objects.create_user(username='joe', password=PASSWORD, email='joe@example.net')
    #     election1 = Election.objects.create(owner=self.user, name='Election', slug='election1', published=True)
    #     election2 = Election.objects.create(owner=self.user, name='Election', slug='election2', published=True)
    #     election3 = Election.objects.create(owner=self.user, name='Election', slug='election3')
    #     election4 = Election.objects.create(owner=self.user, name='Election', slug='election4')
    #     election5 = Election.objects.create(owner=self.user, name='Election', slug='election5')
    #     election6 = Election.objects.create(owner=self.user, name='Election', slug='election6')
    #     self.url = reverse('home')
    #     response = self.client.get(self.url)
    #     self.assertTrue('last_elections' in response.context)
    #     elections = response.context['last_elections']
    #     self.assertTrue(elections.count() == 2)
    #     self.assertTrue(elections[0] == election2)
    #     self.assertTrue('values' in response.context)
    #     self.assertTrue(response.context['values'] == [1,2])
        

class EmbededTestTemplateView(TestCase):

    def test_shows_prueba_html(self):
        url = reverse('prueba_embeded')
        response = self.client.get(url)

        self.assertTemplateUsed(response, 'prueba.html')
        self.assertTrue('embeded_test_web' in response.context)


class UserElectionsViewTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password=PASSWORD, email='joe@example.net')
        self.election1 = Election.objects.create(owner=self.user, name='Election', slug='election1', published=True, highlighted=False)
        election2 = Election.objects.create(owner=self.user, name='Election', slug='election2', published=False, highlighted=False)
        self.url = reverse('user_elections',kwargs={ 'username':self.user.username })

    def test_any_user_can_see_another_users_election(self):


        response = self.client.get(self.url)
        self.assertTemplateUsed(response, 'elections/users_election_list.html')
        self.assertTrue('elections' in response.context)
        self.assertEqual(response.status_code, 200)

        self.assertEqual(response.context['elections'].count(), 1)
        self.assertEqual(response.context['elections'][0], self.election1 )
        self.assertTrue('owner' in response.context)
        self.assertEqual(response.context['owner'], self.user)



class Municipales2012ElectionTemplateView(TestCase):
    def test_renders_a_html_for_this_new_event(self):
        url = reverse('municipales2012')
        response = self.client.get(url)

        self.assertEquals(response.status_code, 200)
        self.assertTemplateUsed(response, 'municipales2012.html')

class TogglePublish(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password=PASSWORD, email='joe@example.net')
        self.user2 = User.objects.create_user(username='maria', password=PASSWORD, email='maria@example.net')
        self.election1 = Election.objects.create(owner=self.user, name='Election', slug='election1', published=False, highlighted=False)

    def test_to_publish_election(self):
        self.client.login(username=self.user.username, password=PASSWORD)
        url = reverse('toggle_publish',kwargs={ 'pk':self.election1.id })
        response = self.client.post(url)
        self.assertEquals(response.status_code, 200)
        self.assertEquals(response['Content-Type'], 'application/json')
        data = json.loads(response.content)
        self.assertEquals(data['id'], self.election1.id)
        self.assertTrue(data['published'])
        election1 = Election.objects.get( id=self.election1.id )
        self.assertTrue(election1.published)

    def test_to_unpublish_election(self):
        self.election1.published = True
        self.election1.save()
        self.client.login(username=self.user.username, password=PASSWORD)
        url = reverse('toggle_publish',kwargs={ 'pk':self.election1.id })
        response = self.client.post(url)
        self.assertEquals(response.status_code, 200)
        self.assertEquals(response['Content-Type'], 'application/json')
        data = json.loads(response.content)
        self.assertEquals(data['id'], self.election1.id)
        self.assertFalse(data['published'])
        election1 = Election.objects.get( id=self.election1.id )
        self.assertFalse(election1.published)

    def test_not_logged_to_publish(self):
        url = reverse('toggle_publish',kwargs={ 'pk':self.election1.id })
        response = self.client.post(url)
        self.assertEquals(response.status_code, 401)

    def test_not_owner_to_publish(self):
        self.client.login(username=self.user2.username, password=PASSWORD)
        url = reverse('toggle_publish',kwargs={ 'pk':self.election1.id })
        response = self.client.post(url)
        self.assertEquals(response.status_code, 403)

    def test_get_to_publish(self):
        self.client.login(username=self.user.username, password=PASSWORD)
        url = reverse('toggle_publish',kwargs={ 'pk':self.election1.id })
        response = self.client.get(url)
        self.assertEquals(response.status_code, 405)

    def test_get_to_publish_if_not_logged(self):
        url = reverse('toggle_publish',kwargs={ 'pk':self.election1.id })
        response = self.client.get(url)
        self.assertEquals(response.status_code, 401)





from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.forms import formsets
from django.http import HttpResponse, Http404
from django.shortcuts import render_to_response, redirect, get_object_or_404
from django.template import RequestContext
from django.template.context import RequestContext
from django.utils import simplejson as json
from django.utils.decorators import method_decorator
from django.views.decorators.http import require_POST
from django.views.generic import CreateView, DetailView, UpdateView

# Import forms
from elections.forms import CategoryForm, CategoryUpdateForm

# Import models
from elections.models import Category, Election


# Category views
class CategoryCreateView(CreateView):
    model = Category
    form_class = CategoryForm

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(CategoryCreateView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(CategoryCreateView, self).get_context_data(**kwargs)
        context['election'] = get_object_or_404(Election, slug=self.kwargs['election_slug'], owner=self.request.user)
        return context

    def get_success_url(self):
        return reverse('category_create', kwargs={'election_slug': self.object.election.slug})

    def form_valid(self, form):
        self.object = form.save(commit=False)
        election = get_object_or_404(Election, slug=self.kwargs['election_slug'], owner=self.request.user)
        self.object.election = election

        try:
            self.object.full_clean()
        except ValidationError:
            from django.forms.util import ErrorList
            form._errors["slug"] = ErrorList([u"Ya tienes una categoria con ese slug."])
            return super(CategoryCreateView, self).form_invalid(form)

        return super(CategoryCreateView, self).form_valid(form)

class CategoryUpdateView(UpdateView):
    model = Category
    form_class = CategoryUpdateForm

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(CategoryUpdateView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(CategoryUpdateView, self).get_context_data(**kwargs)
        context['election'] = get_object_or_404(Election, slug=self.kwargs['election_slug'], owner=self.request.user)
        return context

    def get_success_url(self):
        return reverse('election_detail', kwargs={'slug': self.object.election.slug, 'username': self.request.user.username})

    def get_queryset(self):
        return self.model.objects.filter(election__slug=self.kwargs['election_slug'],
                                         slug=self.kwargs['slug'],
                                         election__owner=self.request.user)

    def form_valid(self, form):
        self.object = form.save(commit=False)
        election = get_object_or_404(Election, slug=self.kwargs['election_slug'], owner=self.request.user)
        self.object.election = election
        return super(CategoryUpdateView, self).form_valid(form)


@login_required
@require_POST
def async_delete_category(request, category_pk):
    category = get_object_or_404(Category, pk = category_pk, election__owner = request.user)
    category.delete()
    json_dictionary = {"result":"OK"}
    return HttpResponse(json.dumps(json_dictionary),content_type='application/json')

@login_required
@require_POST
def async_create_category(request, election_pk):
    election = get_object_or_404(Election, pk=election_pk, owner=request.user)

    value = request.POST.get('value', None)
    category = Category(name=value, election=election)
    try:
        category.full_clean()
    except ValidationError, e:
        errors = {'errors': e.messages}
        return HttpResponse(json.dumps(errors),status=412)

    category.save()
    return HttpResponse(json.dumps({"pk": category.pk, "name": category.name}),
                        content_type='application/json')


# -*- coding: utf-8 -*-
"""
This file demonstrates writing tests using the unittest module. These will pass
when you run "manage.py test".

Replace this with more appropriate tests for your application.
"""

from django.test import TestCase
import slumber
from candideitorg.models import CandideitorgDocument, Election, Category, Candidate, BackgroundCategory,\
                                PersonalData, Background, Answer, Question, PersonalDataCandidate, Link,\
                                BackgroundCandidate, InformationSource
from django.core.management import call_command
from django.utils.unittest import skip
from django.template import Template, Context
from django.utils.translation import ugettext as _
import subprocess
import os
from django.template.loader import get_template
from django.core.management import call_command
from django.db.models import Q
from candideitorg.tasks import election_updater

class CandideitorgTestCase(TestCase):
    @classmethod
    def setUpClass(cls):
        cls.install_candidator_yaml()

    @classmethod
    def install_candidator_yaml(cls, yaml_file='candidator_example_data'):
        FNULL = open(os.devnull, 'w')
        subprocess.call(['./candidator_install_yaml.bash', '../' + yaml_file + ".yaml"], stdout=FNULL, stderr=subprocess.STDOUT)


class CandideitorgMoreThanTwentyElections(CandideitorgTestCase):
    @classmethod
    def setUpClass(cls):
        cls.install_candidator_yaml(yaml_file="candidator_example_data_big")

    def test_load_all_elections(self):
        Election.fetch_all_from_api()
        self.assertEquals(Election.objects.count(), 21)


class UpdatingDataCandidator(CandideitorgTestCase):

    def test_upgrade_data(self):
        Election.fetch_all_from_api()
        Election.fetch_all_from_api()
        self.assertEquals(Election.objects.count(), 1)
        self.assertEquals(Category.objects.count(), 2)
        self.assertEquals(Candidate.objects.count(), 3)
        self.assertEquals(BackgroundCategory.objects.count(),2)
        self.assertEquals(PersonalData.objects.count(),4)
        self.assertEquals(Background.objects.count(),4)
        self.assertEquals(BackgroundCategory.objects.count(),2)

    def test_updates_answer(self):
        UpdatingDataCandidator.install_candidator_yaml()
        Election.fetch_all_from_api()

        UpdatingDataCandidator.install_candidator_yaml(yaml_file='candidator_example_data_with_answers2')
        Election.fetch_all_from_api()

        juanito = Candidate.objects.all()[0]


        paros = Question.objects.get(question='Esta de a cuerdo con los paros?')
        marchas = Question.objects.get(question='Le gusta ir a las marchas?')
        carretear = Question.objects.get(question='Quiere gastar su plata carreteando?')
        plata = Question.objects.get(question='Quiere robarse la plata del CEI?')
        # paros_asnwers = 
        answers_for_paros = paros.answer_set.all()
        number_of_answers_for_paros = 0
        for answer_for_paros in answers_for_paros:
            number_of_answers_for_paros += answer_for_paros.candidate_set.all().count()



        self.assertEquals(juanito.answers.get(question=paros).caption, u'Estoy de acuerdo con algunos paros')
        self.assertEquals(juanito.answers.get(question=marchas).caption, u'Siempre')
        self.assertEquals(juanito.answers.get(question=plata).caption, u'No')
        self.assertEquals(juanito.answers.get(question=carretear).caption, u'A veces')
        self.assertEquals(paros.number_of_answers,1)
        paros.answer_set.all().delete()
        self.assertEquals(paros.number_of_answers,0)
        





    def test_election_update(self):
        Election.fetch_all_from_api()

        UpdatingDataCandidator.install_candidator_yaml(yaml_file='candidator_example_data_with_answers2')

        election = Election.objects.all()[0]
        election.update()

        juanito = Candidate.objects.all()[0]
        self.assertEquals(juanito.answers.all().count(), 4)
        
        
        
        marchas = Question.objects.get(question='Le gusta ir a las marchas?')
        self.assertEquals(juanito.answers.get(question=marchas).caption, u'Siempre')
        
        carretear = Question.objects.get(question='Quiere gastar su plata carreteando?')
        self.assertEquals(juanito.answers.get(question=carretear).caption, u'A veces')
        
        plata = Question.objects.get(question='Quiere robarse la plata del CEI?')
        self.assertEquals(juanito.answers.get(question=plata).caption, u'No')

        paros = Question.objects.get(question='Esta de a cuerdo con los paros?')
        self.assertEquals(juanito.answers.get(question=paros).caption, u'Estoy de acuerdo con algunos paros')


    def test_election_update_with_a_celery_task(self):

        Election.fetch_all_from_api()

        UpdatingDataCandidator.install_candidator_yaml(yaml_file='candidator_example_data_with_answers2')

        election = Election.objects.all()[0]

        result = election_updater.delay(election)

        self.assertTrue(result.successful())

        juanito = Candidate.objects.all()[0]
        self.assertEquals(juanito.answers.all().count(), 4)
        
        
        
        marchas = Question.objects.get(question='Le gusta ir a las marchas?')
        self.assertEquals(juanito.answers.get(question=marchas).caption, u'Siempre')
        
        carretear = Question.objects.get(question='Quiere gastar su plata carreteando?')
        self.assertEquals(juanito.answers.get(question=carretear).caption, u'A veces')
        
        plata = Question.objects.get(question='Quiere robarse la plata del CEI?')
        self.assertEquals(juanito.answers.get(question=plata).caption, u'No')

        paros = Question.objects.get(question='Esta de a cuerdo con los paros?')
        self.assertEquals(juanito.answers.get(question=paros).caption, u'Estoy de acuerdo con algunos paros')




    def test_election_does_not_create_two_personal_data_candidates(self):
        Election.fetch_all_from_api()

        UpdatingDataCandidator.install_candidator_yaml(yaml_file='candidator_example_data_with_answers')

        election = Election.objects.all()[0]

        UpdatingDataCandidator.install_candidator_yaml(yaml_file='candidator_example_data_with_answers2')
        election.update()

        personaldata = election.personaldata_set.get(label=u'Profesion')
        candidate = election.candidate_set.get(name="Juanito Perez")


        personal_data_candidates = personaldata.personaldatacandidate_set.filter(candidate=candidate)
        self.assertEquals(personal_data_candidates.count(), 1)




    def test_election_update_answers(self):
        Election.fetch_all_from_api()
        juanito = Candidate.objects.all()[0]
        
        UpdatingDataCandidator.install_candidator_yaml(yaml_file='candidator_example_data_with_answers2')
        election = Election.objects.all()[0]
        election.update_answers()


        juanito = Candidate.objects.all()[0]

        self.assertEquals(juanito.answers.all().count(), 4)
        
        paros = Question.objects.get(question='Esta de a cuerdo con los paros?')
        self.assertEquals(juanito.answers.get(question=paros).caption, u'Estoy de acuerdo con algunos paros')
        
        marchas = Question.objects.get(question='Le gusta ir a las marchas?')
        self.assertEquals(juanito.answers.get(question=marchas).caption, u'Siempre')
        
        carretear = Question.objects.get(question='Quiere gastar su plata carreteando?')
        self.assertEquals(juanito.answers.get(question=carretear).caption, u'A veces')
        
        plata = Question.objects.get(question='Quiere robarse la plata del CEI?')
        self.assertEquals(juanito.answers.get(question=plata).caption, u'No')

        #now test with new information source
        information_source = InformationSource.objects.get(Q(question=paros) & Q(candidate=juanito))
        self.assertEquals(information_source.content, u'En el diario más cercano')


        tester = Candidate.objects.get(resource_uri='/api/v2/candidate/2/')
        information_source2 = InformationSource.objects.get(Q(question=paros) & Q(candidate=tester))
        self.assertEquals(information_source2.content, u'this IS did not exist before')

        personaldata = election.personaldata_set.get(label=u'Profesion')
        candidate = election.candidate_set.get(name="Juanito Perez")


        personal_data_candidates = personaldata.personaldatacandidate_set.filter(candidate=candidate)
        self.assertEquals(personal_data_candidates.count(), 1)
        self.assertEquals(personal_data_candidates[0].value, u"Grande")


    def test_it_does_not_create_replicate_answers(self):
        Election.fetch_all_from_api()
        UpdatingDataCandidator.install_candidator_yaml(yaml_file='candidator_example_data_updating')
        election = Election.objects.all()[0]
        election.update()
        questions = Question.objects.filter(category__election=election)

        #THERE ARE QUESTIONS ADDED BY DEFAULT TO CANDIDEIT.ORG
        #I HAVE TO FIX THIS IN ORDER TO COUNT
        #4 QUESTIONS AND
        #10 ANSWERS
        self.assertEquals(questions.count(), 6)
        answers = Answer.objects.filter(question__in=questions)

        self.assertEquals(answers.count(), 14)


        candidates = Candidate.objects.filter(election=election)
        self.assertEquals(candidates.count(), 3)




class CandideitorgDocumentTest(TestCase):
    
    def setUp(self):
        super(CandideitorgDocumentTest, self).setUp()

    def test_candideitor_document_is_abstract(self):
        document = CandideitorgDocument()
        
        self.assertTrue(document)
        self.assertTrue(document._meta.abstract)

    def test_candideitorg_document_attribute(self):

        class Element(CandideitorgDocument):
            class Meta:
                app_label = 'candideitorg'
        
        call_command('syncdb', verbosity=0)

        
        element = Element.objects.create(resource_uri="cualquiercosa", remote_id=2)

        self.assertEquals(element.resource_uri,'cualquiercosa')
        self.assertEquals(element.remote_id,2)

    def test_get_api(self):
        class Element(CandideitorgDocument):
            class Meta:
                app_label = 'candideitorg'

        api = Element.get_api()
        self.assertIsInstance(api, slumber.API)


class PossibleNullValues(CandideitorgTestCase):
    def setUp(self):
        super(PossibleNullValues, self).setUp()

    def test_election_null_values(self):
        election = Election.objects.create(
            description = "Elecciones CEI 2012",
            remote_id = 1,
            information_source = None,
            logo = None,
            name = "cei 2012",
            resource_uri = "/api/v2/election/1/",
            slug = "cei-2012"
            )

        self.assertTrue(election)
        self.assertIsNone(election.logo)
        self.assertIsNone(election.information_source)
        self.assertFalse(election.use_default_media_naranja_option)


    def test_candidate_null_values(self):
        self.election = Election.objects.create(
            description = "Elecciones CEI 2012",
            remote_id = 1,
            information_source = "",
            logo = "/media/photos/dummy.jpg",
            name = "cei 2012",
            resource_uri = "/api/v2/election/1/",
            slug = "cei-2012",
            use_default_media_naranja_option = True
            )        
        candidate = Candidate.objects.create(
            name = "Juanito Perez",
            photo = None,
            slug = "juanito-perez",
            has_answered = True,
            election = self.election,
            remote_id = 1
            )

        self.assertTrue(candidate)
        self.assertIsNone(candidate.photo)


    def test_personal_data_candidate_null_values(self):
        self.election = Election.objects.create(
            description = "Elecciones CEI 2012",
            remote_id = 1,
            information_source = "",
            logo = "/media/photos/dummy.jpg",
            name = "cei 2012",
            resource_uri = "/api/v2/election/1/",
            slug = "cei-2012",
            use_default_media_naranja_option = True
            )
        self.candidate = Candidate.objects.create(
            name = "Juanito Perez",
            photo = "/media/photos/dummy.jpg",
            slug = "juanito-perez",
            has_answered = True,
            election = self.election,
            remote_id = 1
            )
        self.personaldata = PersonalData.objects.create(
            label = 'Nacimiento',
            remote_id = 1,
            election = self.election
            )

        personal_data_candidate = PersonalDataCandidate.objects.create(
            remote_id = 1,
            resource_uri = "/api/v2/personal_data_candidate/1/",
            value = None,
            candidate = self.candidate,
            personaldata = self.personaldata
            )

        self.assertTrue(personal_data_candidate)
        self.assertIsNone(personal_data_candidate.value)



class ElectionTest(CandideitorgTestCase):
    def setUp(self):
        super(ElectionTest, self).setUp()

    def test_is_subclass(self):
        election = Election.objects.create(remote_id=2)
        self.assertIsInstance(election,CandideitorgDocument)

    def test_create_election(self):
        election = Election.objects.create(
            description = "Elecciones CEI 2012",
            remote_id = 1,
            information_source = "",
            logo = "/media/photos/dummy.jpg",
            name = "cei 2012",
            resource_uri = "/api/v2/election/1/",
            slug = "cei-2012",
            use_default_media_naranja_option = True
            )

        self.assertTrue(election)
        self.assertEquals(election.description, 'Elecciones CEI 2012')
        self.assertEquals(election.remote_id, 1)
        self.assertEquals(election.information_source, '')
        self.assertEquals(election.logo, '/media/photos/dummy.jpg')
        self.assertEquals(election.name, 'cei 2012')
        self.assertEquals(election.resource_uri, '/api/v2/election/1/')
        self.assertEquals(election.slug, 'cei-2012')
        self.assertTrue(election.use_default_media_naranja_option)

    def test_unicode(self):
        election = Election.objects.create(
            description = "Elecciones CEI 2012",
            remote_id = 1,
            information_source = "",
            logo = "/media/photos/dummy.jpg",
            name = "cei 2012",
            resource_uri = "/api/v2/election/1/",
            slug = "cei-2012",
            use_default_media_naranja_option = True
            )
        self.assertEquals(election.__unicode__(), "cei 2012")

    def test_fetch_all_election(self):
        Election.fetch_all_from_api()
        self.assertEquals(Election.objects.count(), 1)

        election = Election.objects.all()[0]
        self.assertEquals(election.description, 'Elecciones CEI 2012')
        self.assertEquals(election.remote_id, 1)
        self.assertEquals(election.information_source, '')
        self.assertEquals(election.logo, '/media/photos/dummy.jpg')
        self.assertEquals(election.name, 'cei 2012')
        self.assertEquals(election.resource_uri, '/api/v2/election/1/')
        self.assertEquals(election.slug, 'cei-2012')
        self.assertTrue(election.use_default_media_naranja_option)

class CategoryTest(CandideitorgTestCase):

    def setUp(self):
        super(CategoryTest, self).setUp()

    def test_create_category(self):
        election = Election.objects.create(
            description = "Elecciones CEI 2012",
            remote_id = 1,
            information_source = "",
            logo = "/media/photos/dummy.jpg",
            name = "cei 2012",
            resource_uri = "/api/v2/election/1/",
            slug = "cei-2012",
            use_default_media_naranja_option = True
            )

        category = Category.objects.create(
            name='category name',
            election=election,
            slug='category-name',
            order=1,
            resource_uri='/api/v2/category/1/',
            remote_id=1
            )

        self.assertTrue(category)
        self.assertIsInstance(category, CandideitorgDocument)
        self.assertEquals(category.name, 'category name')
        self.assertEquals(category.election, election)
        self.assertEquals(category.slug, 'category-name')
        self.assertEquals(category.order, 1)


    def test_unicode(self):
        election = Election.objects.create(
            description = "Elecciones CEI 2012",
            remote_id = 1,
            information_source = "",
            logo = "/media/photos/dummy.jpg",
            name = "cei 2012",
            resource_uri = "/api/v2/election/1/",
            slug = "cei-2012",
            use_default_media_naranja_option = True
            )

        category = Category.objects.create(
            name='category name',
            election=election,
            slug='category-name',
            order=1,
            resource_uri='/api/v2/category/1/',
            remote_id=1
            )

        expected_unicode = '"category name" in election "cei 2012"'

        self.assertEquals(category.__unicode__(), expected_unicode)

    def test_fetch_all_categories(self):
        Election.fetch_all_from_api()
        election = Election.objects.all()[0]

        self.assertEquals(Category.objects.count(), 2)

        categorie = Category.objects.all()[0]
        self.assertEqual(categorie.remote_id,1)
        self.assertEqual(categorie.name,'Politicas Publicas')
        self.assertEqual(categorie.order,1)
        self.assertEqual(categorie.resource_uri,'/api/v2/category/1/')
        self.assertEquals(categorie.slug,'politicas-publicas')
        self.assertEquals(categorie.election, election)

class CandidatesTest(CandideitorgTestCase):
    def setUp(self):
        super(CandidatesTest, self).setUp()
        self.election = Election.objects.create(
            description = "Elecciones CEI 2012",
            remote_id = 1,
            information_source = "",
            logo = "/media/photos/dummy.jpg",
            name = "cei 2012",
            resource_uri = "/api/v2/election/1/",
            slug = "cei-2012",
            use_default_media_naranja_option = True
            )

    def test_create_candidate(self):
        candidate = Candidate.objects.create(
            name = "Juanito Perez",
            photo = "/media/photos/dummy.jpg",
            slug = "juanito-perez",
            has_answered = True,
            election = self.election,
            remote_id = 1
            )

        self.assertTrue(candidate)
        self.assertIsInstance(candidate, CandideitorgDocument)
        self.assertEquals(candidate.name, 'Juanito Perez')
        self.assertEquals(candidate.photo, '/media/photos/dummy.jpg')
        self.assertEquals(candidate.slug, 'juanito-perez')
        self.assertTrue(candidate.has_answered)
        self.assertEquals(candidate.election, self.election)

    def test_unicode(self):
        candidate = Candidate.objects.create(
            name = "Juanito Perez",
            photo = "/media/photos/dummy.jpg",
            slug = "juanito-perez",
            has_answered = True,
            election = self.election,
            remote_id = 1
            )
        self.assertEquals(candidate.__unicode__(),u"Juanito Perez")

    def test_fetch_all_candidates_from_api(self):
        self.election.delete()
        Election.fetch_all_from_api()
        election = Election.objects.all()[0]

        self.assertEquals(Candidate.objects.count(), 3)

        candidate = Candidate.objects.all()[0]
        self.assertEquals(candidate.remote_id,1)
        self.assertEquals(candidate.name, 'Juanito Perez')
        self.assertEquals(candidate.slug, 'juanito-perez')
        self.assertEquals(candidate.photo, '/media/photos/dummy.jpg')
        self.assertEquals(candidate.election, election)


class InformationSourceTest(CandideitorgTestCase):
    def setUp(self):
        super(InformationSourceTest, self).setUp()
        self.election = Election.objects.create(
            description = "Elecciones CEI 2012",
            remote_id = 1,
            information_source = "",
            logo = "/media/photos/dummy.jpg",
            name = "cei 2012",
            resource_uri = "/api/v2/election/1/",
            slug = "cei-2012",
            use_default_media_naranja_option = True
            )
        self.candidate = Candidate.objects.create(
            name = "Juanito Perez",
            photo = "/media/photos/dummy.jpg",
            slug = "juanito-perez",
            has_answered = True,
            election = self.election,
            resource_uri = '/api/v2/candidate/1/',
            remote_id = 1
            )
        self.category = Category.objects.create(
            name='category name',
            election=self.election,
            slug='category-name',
            order=1,
            resource_uri='/api/v2/category/1/',
            remote_id=1
            )
        self.question = Question.objects.create(
            remote_id = 1,
            question = 'Esta de a cuerdo con los paros?',
            resource_uri = '/api/v2/question/1/',
            category = self.category,
            )
        

    def test_i_can_create_one(self):
        information_source = InformationSource.objects.create(
            question=self.question, 
            candidate=self.candidate,
            content="this is a content",
            remote_id = 1,
            )
        self.assertTrue(information_source)
        self.assertEquals(information_source.question, self.question)
        self.assertEquals(information_source.candidate, self.candidate)
        self.assertEquals(information_source.content, "this is a content")


    def test_pull_from_api_and_relates(self):
        Election.fetch_all_from_api()

        candidate = Candidate.objects.get(resource_uri="/api/v2/candidate/1/")
        question = Question.objects.get(resource_uri="/api/v2/question/1/")

        information_source = InformationSource.objects.all()[0]
        self.assertEquals(information_source.candidate, candidate)
        self.assertEquals(information_source.question, question)
        self.assertEquals(information_source.content, u'Esta de a cuerdo con los paros?')


    def test_unicode(self):
        information_source = InformationSource.objects.create(
            question=self.question, 
            candidate=self.candidate,
            content="this is a content",
            remote_id = 1,
            )
        expected_unicode = u'Reference for "Juanito Perez" in question "Esta de a cuerdo con los paros?"'
        self.assertEquals(information_source.__unicode__(), expected_unicode)

class BackgroundCategoriesTest(CandideitorgTestCase):
    def setUp(self):
        super(BackgroundCategoriesTest, self).setUp()
        self.election = Election.objects.create(
            description = "Elecciones CEI 2012",
            remote_id = 1,
            information_source = "",
            logo = "/media/photos/dummy.jpg",
            name = "cei 2012",
            resource_uri = "/api/v2/election/1/",
            slug = "cei-2012",
            use_default_media_naranja_option = True
            )

    def test_create_background_categorie(self):
        bg_categorie = BackgroundCategory.objects.create(
            name = 'Tendencia Politica',
            resource_uri = "/api/v2/background_category/1/",
            election=self.election,
            remote_id=1,
            )

        self.assertTrue(bg_categorie)
        self.assertIsInstance(bg_categorie, CandideitorgDocument)
        self.assertEquals(bg_categorie.name, 'Tendencia Politica')
        self.assertEquals(bg_categorie.election, self.election)

    def test_fetch_all_background_candidates_from_api(self):
        self.election.delete()
        Election.fetch_all_from_api()
        election = Election.objects.all()[0]

        self.assertEquals(BackgroundCategory.objects.count(),2)

        bg_categorie = BackgroundCategory.objects.all()[0]
        self.assertEquals(bg_categorie.remote_id,1)
        self.assertEquals(bg_categorie.name,'Tendencia Politica')
        self.assertEquals(bg_categorie.election,election)

class PersonalDataTest(CandideitorgTestCase):
    def setUp(self):
        super(PersonalDataTest, self).setUp()
        self.election = Election.objects.create(
            description = "Elecciones CEI 2012",
            remote_id = 1,
            information_source = "",
            logo = "/media/photos/dummy.jpg",
            name = "cei 2012",
            resource_uri = "/api/v2/election/1/",
            slug = "cei-2012",
            use_default_media_naranja_option = True
            )

    def test_create_personaldata(self):

        personaldata = PersonalData.objects.create(
            label = 'Nacimiento',
            remote_id = 1,
            election = self.election
            )

        self.assertTrue(personaldata)
        self.assertIsInstance(personaldata, CandideitorgDocument)
        self.assertEquals(personaldata.label,'Nacimiento')
        self.assertEquals(personaldata.election, self.election)

    def test_fetch_all_personaldata_from_api(self):
        self.election.delete()
        Election.fetch_all_from_api()
        election = Election.objects.all()[0]

        self.assertEquals(PersonalData.objects.count(),4)

        personal_data = PersonalData.objects.all()[0]
        self.assertEquals(personal_data.label,'Nacimiento')
        self.assertEquals(personal_data.election, election)

class BackgroundTest(CandideitorgTestCase):
    def setUp(self):
        super(BackgroundTest, self).setUp()
        self.election = Election.objects.create(
            description = "Elecciones CEI 2012",
            remote_id = 1,
            information_source = "",
            logo = "/media/photos/dummy.jpg",
            name = "cei 2012",
            resource_uri = "/api/v2/election/1/",
            slug = "cei-2012",
            use_default_media_naranja_option = True
            )
        self.background_category = BackgroundCategory.objects.create(
            name = 'Tendencia Politica',
            resource_uri = "/api/v2/background_category/1/",
            election=self.election,
            remote_id=1,
            )

    def test_create_background(self):
        background = Background.objects.create(
            remote_id = 1,
            name = 'Partido politico actual',
            resource_uri = '/api/v2/background/1/',
            background_category = self.background_category
            )

        self.assertTrue(background)
        self.assertIsInstance(background, CandideitorgDocument)
        self.assertEquals(background.name, 'Partido politico actual')
        self.assertEquals(background.resource_uri, '/api/v2/background/1/')
        self.assertEquals(background.background_category, self.background_category)

    def test_fetch_all_background_from_api(self):
        self.election.delete()
        Election.fetch_all_from_api()
        election = Election.objects.all()[0]
        background_category = BackgroundCategory.objects.get(remote_id=1)

        self.assertEquals(Background.objects.count(),4)
        background = Background.objects.get(remote_id=1)
        self.assertEquals(background.name,'Partido politico actual')
        self.assertEquals(background.resource_uri,'/api/v2/background/1/')
        self.assertEquals(background.background_category, background_category)

class AnswersTest(CandideitorgTestCase):
    def setUp(self):
        super(AnswersTest, self).setUp()
        self.election = Election.objects.create(
            description = "Elecciones CEI 2012",
            remote_id = 1,
            information_source = "",
            logo = "/media/photos/dummy.jpg",
            name = "cei 2012",
            resource_uri = "/api/v2/election/1/",
            slug = "cei-2012",
            use_default_media_naranja_option = True
            )
        self.category = Category.objects.create(
            name='category name',
            election=self.election,
            slug='category-name',
            order=1,
            resource_uri='/api/v2/category/1/',
            remote_id=1
            )
        self.question = Question.objects.create(
            remote_id = 1,
            question = 'Esta de a cuerdo con los paros?',
            resource_uri = '/api/v2/question/1/',
            category = self.category,
            )
        self.candidate = Candidate.objects.create(
            name = "Juanito Perez",
            photo = "/media/photos/dummy.jpg",
            slug = "juanito-perez",
            has_answered = True,
            election = self.election,
            resource_uri = '/api/v2/candidate/1/',
            remote_id = 1
            )

    def test_unicode(self):
        answer = Answer.objects.create(
            remote_id = 1,
            caption = 'De vez en cuando',
            resource_uri = '/api/v2/answer/8/',
            question = self.question
            )

        self.assertEquals(answer.__unicode__(), "'De vez en cuando' for 'Esta de a cuerdo con los paros?'")

    def test_create_answer(self):
        answer = Answer.objects.create(
            remote_id = 1,
            caption = 'De vez en cuando',
            resource_uri = '/api/v2/answer/8/',
            question = self.question
            )
        self.assertTrue(answer)
        self.assertIsInstance(answer, CandideitorgDocument)
        self.assertEquals(answer.caption, 'De vez en cuando')
        self.assertEquals(answer.resource_uri, '/api/v2/answer/8/')
        self.assertEquals(answer.question, self.question)

    def test_fetch_all_answers_from_api(self):
        self.election.delete()
        Election.fetch_all_from_api()
        election = Election.objects.all()[0]
        category = Category.objects.all()[0]
        question = Question.objects.all()[0]

        self.assertEquals(Answer.objects.count(),10)
        answer = Answer.objects.get(remote_id=9)
        self.assertEquals(answer.caption,'Puro perder clases')
        self.assertEquals(answer.resource_uri,'/api/v2/answer/9/')
        self.assertEquals(answer.question, question)

    def test_get_candidates_has_answer(self):
        candidate = self.candidate
        answer = Answer.objects.create(
            remote_id = 1,
            caption = 'De vez en cuando',
            resource_uri = '/api/v2/answer/8/',
            question = self.question
            )
        candidate.answers.add(answer)
        candidate.save()

        self.assertEquals(candidate.answers.all().count(),1)
        self.assertEquals(candidate.answers.all()[0],answer)

    def test_associate_answers(self):
        # Candidate.objects.all().delete()
        Election.fetch_all_from_api()
        candidate1 = Candidate.objects.get(resource_uri="/api/v2/candidate/1/")
        answer8 = Answer.objects.get(resource_uri="/api/v2/answer/8/")

        self.assertIn(answer8, candidate1.answers.all())

class QuestionTest(CandideitorgTestCase):
    def setUp(self):
        super(QuestionTest, self).setUp()
        self.election = Election.objects.create(
            description = "Elecciones CEI 2012",
            remote_id = 1,
            information_source = "",
            logo = "/media/photos/dummy.jpg",
            name = "cei 2012",
            resource_uri = "/api/v2/election/1/",
            slug = "cei-2012",
            use_default_media_naranja_option = True
            )
        self.category = Category.objects.create(
            name='category name',
            election=self.election,
            slug='category-name',
            order=1,
            resource_uri='/api/v2/category/1/',
            remote_id=1
            )

    def test_create_question(self):
        question = Question.objects.create(
            remote_id = 1,
            question = 'Le gusta ir a las marchas?',
            resource_uri = '/api/v2/question/2/',
            category = self.category,
            )
        self.assertTrue(question)
        self.assertIsInstance(question, CandideitorgDocument)
        self.assertEquals(question.question, 'Le gusta ir a las marchas?')
        self.assertEquals(question.resource_uri, '/api/v2/question/2/')

    def test_unicode(self):
        question = Question.objects.create(
            remote_id = 1,
            question = 'Le gusta ir a las marchas?',
            resource_uri = '/api/v2/question/2/',
            category = self.category,
            )

        expected_unicode = u'"Le gusta ir a las marchas?" in category "category name", in election "cei 2012"'
        self.assertEquals(question.__unicode__(), expected_unicode)


    def test_get_all_question_from_api(self):
        self.election.delete()
        Election.fetch_all_from_api()
        election = Election.objects.all()[0]
        category = Category.objects.all()[0]

        self.assertEquals(Question.objects.count(),4)
        question = Question.objects.all()[0]
        self.assertEquals(question.question, 'Esta de a cuerdo con los paros?')
        self.assertEquals(question.category, category)
        self.assertEquals(question.resource_uri, '/api/v2/question/1/')

class TemplateTagsTest(CandideitorgTestCase):
    def setUp(self):
        super(TemplateTagsTest, self).setUp()
        self.election = Election.fetch_all_from_api()

    def test_templatetag_candidate_answer_value(self):
        question = Question.objects.get(resource_uri="/api/v2/question/2/")
        candidate = Candidate.objects.get(resource_uri="/api/v2/candidate/1/")
        answer = Answer.objects.get(resource_uri="/api/v2/answer/8/")

        expected_html = answer.caption

        template = Template("{% load candideitorg_templetags %}{% answer_for_candidate_and_question candidate question %}")
        context = Context({'candidate':candidate,'question':question})

        self.assertEquals(template.render(context),expected_html)

    def test_template_tag_when_no_answer_is_selected(self):
        candidate = Candidate.objects.get(resource_uri="/api/v2/candidate/1/")
        question = Question.objects.get(resource_uri="/api/v2/question/3/")
        #No one has answered the question 3
        candidate.answers.all().delete()
        #No one has answered the question 3

        expected_html = _(u"Aún no hay respuesta")
        template = Template("{% load candideitorg_templetags %}{% answer_for_candidate_and_question candidate question %}")
        context = Context({'candidate':candidate,'question':question})


        self.assertEquals(template.render(context),expected_html)

    def test_templatetag_personal_data(self):
        candidate = Candidate.objects.get(resource_uri="/api/v2/candidate/1/")
        personal_data = PersonalData.objects.get(resource_uri="/api/v2/personal_data/1/")
        personal_data_candidate = PersonalDataCandidate.objects.get(resource_uri="/api/v2/personal_data_candidate/1/")
        expected_html = personal_data_candidate.value

        template = Template("{% load candideitorg_templetags %}{% relation_personal_data_candidate candidate personal_data %}")
        context = Context({'candidate':candidate,'personal_data':personal_data})

        self.assertEquals(template.render(context),expected_html)

    def test_information_source(self):
        candidate = Candidate.objects.get(resource_uri="/api/v2/candidate/1/")
        question = Question.objects.get(resource_uri="/api/v2/question/1/")
        information_source, created = InformationSource.objects.get_or_create(question=question, candidate=candidate, remote_id=1)
    
        information_source.content = 'olinwi'
        information_source.save()

        template = Template("{% load candideitorg_templetags %}{% get_information_source candidate question %}")
        context = Context({'candidate':candidate,'question':question})
        htmly     = get_template('candideitorg/information_source.html')
        html_content = htmly.render(Context({'information_source': information_source}))
        self.assertEquals(template.render(context),html_content)

    def test_get_empty_information_source(self):
        candidate = Candidate.objects.get(resource_uri="/api/v2/candidate/1/")
        question = Question.objects.get(resource_uri="/api/v2/question/2/")
        #I will not create any information_source
    
        template = Template("{% load candideitorg_templetags %}{% get_information_source candidate question %}")
        context = Context({'candidate':candidate,'question':question})

        self.assertFalse(template.render(context))

class PersonalDataCandidateTest(CandideitorgTestCase):
    def setUp(self):
        super(PersonalDataCandidateTest, self).setUp()
        self.election = Election.objects.create(
            description = "Elecciones CEI 2012",
            remote_id = 1,
            information_source = "",
            logo = "/media/photos/dummy.jpg",
            name = "cei 2012",
            resource_uri = "/api/v2/election/1/",
            slug = "cei-2012",
            use_default_media_naranja_option = True
            )
        self.candidate = Candidate.objects.create(
            name = "Juanito Perez",
            photo = "/media/photos/dummy.jpg",
            slug = "juanito-perez",
            has_answered = True,
            election = self.election,
            remote_id = 1
            )
        self.personaldata = PersonalData.objects.create(
            label = 'Nacimiento',
            remote_id = 1,
            election = self.election
            )

    def test_create_personal_data_candidate(self):
        personal_data_candidate = PersonalDataCandidate.objects.create(
            remote_id = 1,
            resource_uri = "/api/v2/personal_data_candidate/1/",
            value = "13/13/13",
            candidate = self.candidate,
            personaldata = self.personaldata
            )
        self.assertTrue(personal_data_candidate)
        self.assertIsInstance(personal_data_candidate, CandideitorgDocument)
        self.assertEquals(personal_data_candidate.value,'13/13/13')
        self.assertEquals(personal_data_candidate.resource_uri, '/api/v2/personal_data_candidate/1/')
        self.assertEquals(personal_data_candidate.candidate, self.candidate)
        self.assertEquals(personal_data_candidate.personaldata, self.personaldata)

    def test_get_all_personal_data_candidate_from_api(self):
        self.election.delete()
        Election.fetch_all_from_api()

        election = Election.objects.all()[0]
        candidate = Candidate.objects.all()[0]
        personal_data = PersonalData.objects.all()[0]

        self.assertEquals(PersonalDataCandidate.objects.count(),2)
        personal_data_candidate = PersonalDataCandidate.objects.all()[0]
        self.assertEquals(personal_data_candidate.value,'13/13/13')
        self.assertEquals(personal_data_candidate.candidate, candidate)
        self.assertEquals(personal_data_candidate.personaldata, personal_data)

class LinkTest(CandideitorgTestCase):
    def setUp(self):
        super(LinkTest,self).setUp()
        self.election = Election.objects.create(
            description = "Elecciones CEI 2012",
            remote_id = 1,
            information_source = "",
            logo = "/media/photos/dummy.jpg",
            name = "cei 2012",
            resource_uri = "/api/v2/election/1/",
            slug = "cei-2012",
            use_default_media_naranja_option = True
            )
        self.candidate = Candidate.objects.create(
            name = "Juanito Perez",
            photo = "/media/photos/dummy.jpg",
            slug = "juanito-perez",
            has_answered = True,
            election = self.election,
            remote_id = 1
            )

    def test_create_link(self):
        link = Link.objects.create(
            name = 'twitter',
            url = 'http://www.twitter.com',
            candidate = self.candidate,
            remote_id = 1
            )
        self.assertTrue(link)

    def test_get_links_from_api(self):
        self.election.delete()
        Election.fetch_all_from_api()

        election = Election.objects.all()[0]
        candidate = Candidate.objects.all()[0]
        link = Link.objects.all()[0]
        
        self.assertEquals(Link.objects.count(),1)
        self.assertEquals(link.url, 'http://www.twitter.com')

    def test_get_links_extra_classes_for_twitter(self):
        link = Link.objects.create(
            name = 'twitter',
            url = 'http://www.twitter.com',
            candidate = self.candidate,
            remote_id = 1
            )

        expected_extra_links = 'icon-twitter-sign'
        self.assertEquals(link.icon_class, expected_extra_links)

    def test_get_links_extra_classes_for_facebook(self):
        link = Link.objects.create(
            name = 'mi_feisBuuK',
            url = 'http://www.facebook.com',
            candidate = self.candidate,
            remote_id = 1
            )

        expected_extra_links = 'icon-facebook-sign'
        self.assertEquals(link.icon_class, expected_extra_links)

class BackgroundCandidateTest(CandideitorgTestCase):
    def setUp(self):
        super(BackgroundCandidateTest, self).setUp()
        Election.fetch_all_from_api()

    def test_create_background_candidate(self):
        background = Background.objects.get(id=1)
        candidate = Candidate.objects.get(id= 1)
        background_candidate = BackgroundCandidate.objects.create(
            value = 'CCC',
            remote_id = 1,
            candidate = candidate,
            background = background
            )
        self.assertTrue(background_candidate)
        self.assertIsInstance(background_candidate, CandideitorgDocument)
        self.assertEquals(background_candidate.value,'CCC')

    def test_get_all_background_candidates_from_api(self):
        candidate = Candidate.objects.get(resource_uri="/api/v2/candidate/1/")
        background = Background.objects.get(resource_uri="/api/v2/background/1/")

        backgrounds_candidates = BackgroundCandidate.objects.filter(
            resource_uri="/api/v2/backgrounds_candidate/1/")
        self.assertEquals(backgrounds_candidates.count(), 1)
        background_candidate = backgrounds_candidates[0]
        self.assertEquals(background_candidate.candidate, candidate)
        self.assertEquals(background_candidate.background, background)

from candideitorg.models import election_finished
from django.dispatch import receiver

# defininig the proxy according to 
# http://stackoverflow.com/questions/3829742/assert-that-a-method-was-called-in-a-python-unit-test

class MethodCallLogger(object):
   def __init__(self, method):
     self.method = method
     self.was_called = False
     self.times = 0

   def __call__(self, sender=None, **kwargs):
     self.times += 1
     self.method(sender, **kwargs)
     self.was_called = True
     

class SignalAfterAllTestCase(CandideitorgTestCase):
    def setUp(self):
        super(SignalAfterAllTestCase, self).setUp()

    def test_a_signal_is_called(self):

        #defining the receiver
        def signal_receiver(sender, instance, created, **kwargs):
            self.assertIsInstance(instance, Election)
            self.assertEquals(sender, Election)
            self.assertTrue(created)


        signal_receiver = MethodCallLogger(signal_receiver)
        #connecting the thing
        election_finished.connect(signal_receiver)

        Election.fetch_all_from_api()

        self.assertTrue(signal_receiver.was_called)
        self.assertEquals(signal_receiver.times, 1)

    def test_it_send_created_parameter_depending_for_updating(self):
        #defining the receiver
        def signal_receiver(sender, instance, created, **kwargs):
            if signal_receiver.times == 1:
                self.assertTrue(created)
            if signal_receiver.times == 2:
                self.assertFalse(created)


        signal_receiver = MethodCallLogger(signal_receiver)
        #connecting the thing
        election_finished.connect(signal_receiver)
        #creating
        Election.fetch_all_from_api()
        #updating
        Election.fetch_all_from_api()


class ManagementCommandTestCase(CandideitorgTestCase):
    def test_call_command(self):
        self.assertEquals(Election.objects.count(), 0)
        call_command('update_from_candidator')
        self.assertEquals(Election.objects.count(), 1)


class ElectionFetchAllFromAPIPagination(CandideitorgTestCase):
    @classmethod
    def setUpClass(cls):
        cls.install_candidator_yaml(yaml_file="candidator_example_data_big")\

    def test_it_gets_only_the_first_5_elections(self):
        Election.fetch_all_from_api(max_elections=5, offset=0)
        self.assertEquals(Election.objects.count(), 5)


# encoding=UTF-8
from django.core.files.base import File
import json
import os
from django.conf import settings
from django.test import TestCase
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.db import IntegrityError
from elections.forms.election_form import AnswerForm
from django.template import Template, Context
from django.utils.translation import ugettext as _

from django.core.urlresolvers import resolve

from elections.models import Election, Candidate, Category, PersonalData, BackgroundCategory, Background, PersonalDataCandidate, Question, Answer
from elections.forms import ElectionForm, ElectionUpdateForm, PersonalDataForm, BackgroundCategoryForm, BackgroundForm, QuestionForm, CategoryForm, ElectionLogoUpdateForm
from elections.views import ElectionRedirectView, ElectionDetailView

dirname = os.path.dirname(os.path.abspath(__file__))



class ElectionEmbededDetail(TestCase):
	def setUp(self):
		self.user = User.objects.create(username='foobar')
		self.election = Election.objects.create(name='elec foo', slug='elec-foo', owner=self.user, published=True)
		#Deleting default categories
		for category in self.election.category_set.all():
			category.delete()
		#end of deleting
		f = open(os.path.join(dirname, 'media/dummy.jpg'), 'rb')
		self.candidate_one = Candidate.objects.create(name='bar baz', election = self.election, photo=File(f))
		self.candidate_two = Candidate.objects.create(name='foo fii', election = self.election, photo=File(f))
		self.category1, created = Category.objects.get_or_create(name='FooCat',
																election=self.election,
																slug='foo-cat')
		self.category2, created = Category.objects.get_or_create(name='FooCat2',
																election=self.election,
																slug='foo-cat-2')
		self.question1, created = Question.objects.get_or_create(question='FooQuestion',
																category=self.category1)
		self.question2, created = Question.objects.get_or_create(question='BarQuestion',
																category=self.category2)
		self.answer1_1, created = Answer.objects.get_or_create(question=self.question1,
																caption='BarAnswer1Question1')
		self.answer1_2, created = Answer.objects.get_or_create(question=self.question2,
																caption='BarAnswer1Question2')
		self.answer2_1, created = Answer.objects.get_or_create(question=self.question1,
																caption='BarAnswer2uestion1')
		self.answer2_2, created = Answer.objects.get_or_create(question=self.question2,
																caption='BarAnswer2Question2')

		self.candidate_one.associate_answer(self.answer1_1)
		self.candidate_one.associate_answer(self.answer1_2)
		self.candidate_two.associate_answer(self.answer2_1)
		self.candidate_two.associate_answer(self.answer2_2)

	def test_detail_embeded_view_url(self):
		
		url = reverse('election_detail_embeded',kwargs={'username': self.user.username,'slug': self.election.slug})
		response = self.client.get(url)
		self.assertEquals(response.status_code, 200)
		self.assertTemplateUsed(response, "elections/embeded/base_embed.html")
		self.assertTemplateNotUsed(response, "elections/election_detail.html")
		self.assertTemplateUsed(response, "elections/embeded/election_detail_profiles.html")

		self.assertTrue('election' in response.context)
		self.assertEquals(response.context['election'], self.election)

		resolver_match = resolve(url)
		view = ElectionDetailView.as_view()

		self.assertEquals(resolver_match.func.__module__,view.__module__)
		self.assertEquals(resolver_match.func.__name__,view.__name__)





	def test_profiles_view(self):
		url = reverse('election_detail_profiles_embeded',kwargs={'username': self.user.username,'slug': self.election.slug})
		response = self.client.get(url)
		self.assertEquals(response.status_code, 200)
		self.assertTemplateUsed(response, "elections/embeded/base_embed.html")
		self.assertTemplateNotUsed(response, "elections/election_detail_profiles.html")
		self.assertTrue('election' in response.context)
		self.assertEquals(response.context['election'], self.election)
		resolver_match = resolve(url)
		view = ElectionDetailView.as_view()

		self.assertEquals(resolver_match.func.__module__,view.__module__)
		self.assertEquals(resolver_match.func.__name__,view.__name__)

	def test_medianaranja_view(self):
		url = reverse('medianaranja1_embeded',kwargs={'username': self.user.username,'election_slug': self.election.slug})

		response = self.client.get(url)
		self.assertEqual(response.status_code, 200)

		self.assertTrue('stt' in response.context)
		self.assertTrue('election' in response.context)
		self.assertEqual(self.election, response.context['election'])
		
		self.assertTemplateNotUsed(response, "medianaranja1.html")
		self.assertTemplateUsed(response, "elections/embeded/medianaranja1.html")
		self.assertTemplateUsed(response, "elections/embeded/base_embed.html")

		resolver_match = resolve(url)
		self.assertEquals(resolver_match.func.__module__,"candidator.elections.views.medianaranja_views")


	def test_medianaranja_to_answer_view(self):
		answers = [self.answer1_1.pk, self.answer1_2.pk]
		questions_ids = [self.answer1_1.question.pk, self.answer1_2.question.pk]
		importances = [5,3]
		importances_by_category = [5,3]
		factor_question1 = ( answers[0] == self.answer1_1.pk) * importances[0]
		factor_question2 = ( answers[1] == self.answer1_2.pk) * importances[1]
		score_category1 = factor_question1 * 100.0 / importances_by_category[0]
		score_category2 = factor_question2 * 100.0 / importances_by_category[1]
		global_score = (factor_question1 + factor_question2) * 100.0 / sum(importances_by_category)
		url = reverse('medianaranja1_embeded',kwargs={'username': self.user.username,'election_slug': self.election.slug})
		response = self.client.post(url, {'question-0': answers[0], 'question-1': answers[1], \
			'importance-0': importances[0], 'importance-1': importances[1],\
			'question-id-0': questions_ids[0], 'question-id-1': questions_ids[1]})
		expected_winner = [global_score, [score_category1, score_category1], self.candidate_one]
		self.assertEqual(response.context['winner'], expected_winner)
		self.assertEqual(response.status_code, 200)
		self.assertTemplateNotUsed(response, "medianaranja2.html")
		self.assertTemplateUsed(response, "elections/embeded/medianaranja2.html")
		self.assertTemplateUsed(response, "elections/embeded/base_embed.html")

		resolver_match = resolve(url)
		self.assertEquals(resolver_match.func.__module__,"candidator.elections.views.medianaranja_views")

	def test_medianaranja_embeded_does_not_require_csrf_token(self):
		from django.test import Client
		csrf_client = Client(enforce_csrf_checks=True)

		answers = [self.answer1_1.pk, self.answer1_2.pk]
		questions_ids = [self.answer1_1.question.pk, self.answer1_2.question.pk]
		importances = [5,3]
		importances_by_category = [5,3]
		factor_question1 = ( answers[0] == self.answer1_1.pk) * importances[0]
		factor_question2 = ( answers[1] == self.answer1_2.pk) * importances[1]
		score_category1 = factor_question1 * 100.0 / importances_by_category[0]
		score_category2 = factor_question2 * 100.0 / importances_by_category[1]
		global_score = (factor_question1 + factor_question2) * 100.0 / sum(importances_by_category)
		url = reverse('medianaranja1_embeded',kwargs={'username': self.user.username,'election_slug': self.election.slug})
		response = csrf_client.post(url, {'question-0': answers[0], 'question-1': answers[1], \
			'importance-0': importances[0], 'importance-1': importances[1],\
			'question-id-0': questions_ids[0], 'question-id-1': questions_ids[1]})


		self.assertEqual(response.status_code, 200)



	def test_compare_view(self):
		url = reverse('election_compare_embeded',kwargs={'username': self.user.username,'slug': self.election.slug})

		response = self.client.get(url)
		self.assertEqual(response.status_code, 200)
		self.assertTemplateNotUsed(response, "elections/election_compare.html")
		self.assertTemplateUsed(response, "elections/embeded/election_compare.html")
		self.assertTemplateUsed(response, "elections/embeded/base_embed.html")


	def test_compare_view_with_two_candidates_and_category(self):
		url = reverse('election_compare_two_candidates_embeded',kwargs={
			'username': self.user.username,
			'slug': self.election.slug,
			'first_candidate_slug':self.candidate_one.slug,
			'second_candidate_slug':self.candidate_two.slug,
			'category_slug':self.category1.slug

			})

		response = self.client.get(url)

		self.assertEquals(response.status_code, 200)
		self.assertTemplateNotUsed(response, "elections/election_compare.html")
		self.assertTemplateUsed(response, "elections/embeded/election_compare.html")
		self.assertTemplateUsed(response, "elections/embeded/base_embed.html")


	def test_compare_view_with_two_candidates_and_no_category(self):
		url = reverse('election_compare_two_candidates_and_no_category_embeded',kwargs={
			'username': self.user.username,
			'slug': self.election.slug,
			'first_candidate_slug':self.candidate_one.slug,
			'second_candidate_slug':self.candidate_two.slug

			})

		response = self.client.get(url)

		self.assertEquals(response.status_code, 200)
		self.assertTemplateNotUsed(response, "elections/election_compare.html")
		self.assertTemplateUsed(response, "elections/embeded/election_compare.html")
		self.assertTemplateUsed(response, "elections/embeded/base_embed.html")

	def test_compare_view_with_one_candidate_no_category(self):
		url = reverse('election_compare_one_candidate_embeded',kwargs={
			'username': self.user.username,
			'slug': self.election.slug,
			'first_candidate_slug':self.candidate_one.slug

			})
		response = self.client.get(url)

		self.assertEquals(response.status_code, 200)
		self.assertTemplateNotUsed(response, "elections/election_compare.html")
		self.assertTemplateUsed(response, "elections/embeded/election_compare.html")
		self.assertTemplateUsed(response, "elections/embeded/base_embed.html")


	def test_display_candidate_profile(self):
		url = reverse('candidate_detail_embeded',kwargs={
			'username': self.user.username,
			'election_slug': self.election.slug,
			'slug':self.candidate_one.slug
			})
		response = self.client.get(url)
		self.assertEquals(response.status_code, 200)
		self.assertTemplateNotUsed(response, "elections/candidate_detail.html")
		self.assertTemplateUsed(response, "elections/embeded/candidate_detail.html")
		self.assertTemplateUsed(response, "elections/embeded/base_embed.html")


	def test_display_election_about(self):
		url = reverse('election_about_embeded',kwargs={
			'username': self.user.username,
			'slug': self.election.slug
			})
		response = self.client.get(url)
		self.assertEquals(response.status_code, 200)
		self.assertTemplateUsed(response, "elections/embeded/base_embed.html")
		self.assertTemplateUsed(response, "elections/embeded/about.html")
		self.assertTemplateNotUsed(response, "elections/election_about.html")
		self.assertTemplateNotUsed(response, "elections/base.html")














#encoding=UTF-8
from django import template
from django.utils.translation import ugettext as _
from elections.models import Answer

register = template.Library()


@register.simple_tag
def answer_for_candidate_and_question(candidate, question):
    '''
    Returns the answer for the given candidate and question pair.

    >> {% answer_for_candidate_and_question candidate question %}
    "answer"
    '''

    try:
        return question.answer_set.get(candidate=candidate).caption
    except Answer.DoesNotExist:
        pass
    return _(u"Aún no hay respuesta")
    
    
@register.simple_tag
def link_to_updating_this_election(user, election):
    from django.core.urlresolvers import reverse
    is_the_owner = not (user is None) and (election.owner.username == user.username)
    
    if is_the_owner:
        return '<span class="goedit"><a href="'+reverse('election_update',kwargs={"slug":election.slug})+'">'+_(u"Editar Elección")+'</a></span>'
    return ''



# -*- coding: utf-8 -*-
from django.test import TestCase
from django.contrib.auth.models import User
from django.db import models
from elections.models import Election, Candidate, Category, PersonalData, \
                             BackgroundCategory, Background, PersonalDataCandidate, \
                             Question, Answer, Link, BackgroundCandidate, InformationSource
from elections.api import BackgroundResource
from tastypie.test import ResourceTestCase, TestApiClient
from django.core import serializers
from django.core.urlresolvers import reverse
from django.utils.unittest import skip
from django.contrib.sites.models import Site

class ApiV2TestCase(ResourceTestCase):
    def setUp(self):
        super(ApiV2TestCase, self).setUp()
        self.user = User.objects.create_user(username='the_user',
                                                password='joe',
                                                email='doe@joe.cl')

        self.user2 = User.objects.create_user(username='the_user_2',
                                                password='joe_2',
                                                email='doe@joe.cl')
        # delete all elections
        Election.objects.filter(owner=self.user).delete()

        #create an election
        self.election, created = Election.objects.get_or_create(name='Election user 1',
                                                            owner=self.user,
                                                            slug='name-of-election',
                                                            published=True)

        self.election2, created = Election.objects.get_or_create(name='Election user 2',
                                                            owner=self.user2,
                                                            slug='election-user-2',
                                                            published=True)
        #delete category created by default
        self.election.category_set.all().delete()
        self.election2.category_set.all().delete()

        self.personal_data1 = PersonalData.objects.create(
            label = u"Age", 
            election = self.election)

        self.personal_data2 = PersonalData.objects.create(
            label = u"Le Profession", 
            election = self.election)

        self.candidate = Candidate.objects.create(
            name = u"Le Candidate 01",
            election = self.election
            )

        self.candidate2 = Candidate.objects.create(
            name = u"Candidate 02",
            election = self.election
            )

        self.category1 = Category.objects.create(
            name = u"Category 01", 
            election = self.election, 
            slug = "category-01")

        self.category2 = Category.objects.create(
            name = u"Another category", 
            election = self.election, 
            slug = "another-category")

        self.question_category_1 = Question.objects.create(
            category = self.category1,
            question = u"Te gusta the beatles?")

        self.question_category_2 = Question.objects.create(
            category = self.category2,
            question = u"Comerías limón con ají?")

        self.answer_for_question_1 = Answer.objects.create(
            question = self.question_category_1,
            caption = u"Si")

        self.answer_for_question_3 = Answer.objects.create(
            question = self.question_category_1,
            caption = u"No")

        self.answer_for_question_2 = Answer.objects.create(
            question = self.question_category_2,
            caption = u"Talvez")

        self.answer_for_question_4 = Answer.objects.create(
            question = self.question_category_2,
            caption = u"Quizás")

        self.age = PersonalDataCandidate.objects.create(
            personal_data = self.personal_data1, 
            candidate = self.candidate, 
            value = u"31")

        self.profession = PersonalDataCandidate.objects.create(
            personal_data = self.personal_data2, 
            candidate = self.candidate, 
            value = u"Constructor")

        self.background_category_education = BackgroundCategory.objects.create(
            name = u"Education",
            election = self.election
            )

        self.background_school = Background.objects.create(
            name = u"School",
            category = self.background_category_education
            )

        self.background_candidate = BackgroundCandidate.objects.create(
            candidate = self.candidate,
            background = self.background_school,
            value = u"Primary High School Musical"
            )

        self.link_twitter = Link.objects.create(
            name = u"Twitter",
            url = u"https://www.twitter.com/#el_twitter",
            candidate = self.candidate)

        self.candidate.associate_answer(self.answer_for_question_1)
        self.candidate.associate_answer(self.answer_for_question_2)

        
        self.api_client = TestApiClient()
        #self.data = {'format': 'json', 'username': self.user.username, 'api_key':self.user.api_key.key}

    def get_credentials(self):
        return self.create_apikey(username=self.user.username, api_key=self.user.api_key.key)

    def test_get_all_my_elections_unauthorized(self):
        response = self.api_client.get('/api/v2/election/', format='json')
        self.assertHttpUnauthorized(response)

    def test_get_elections_valid_json(self):
        response = self.api_client.get(
            '/api/v2/election/', 
            format='json', 
            authentication=self.get_credentials())
        self.assertValidJSONResponse(response)

    def test_get_my_elections(self):
        response = self.api_client.get(
            '/api/v2/election/', 
            format='json', 
            authentication=self.get_credentials())
        elections = self.deserialize(response)['objects']
        self.assertEquals(len(elections),1)
        the_election = elections[0]
        # print the_election["background"]
        self.assertEquals(the_election['id'],self.election.id)
        self.assertTrue(the_election.has_key('candidates'))
        self.assertTrue(isinstance(the_election["candidates"][0], unicode))
        self.assertTrue(the_election.has_key('categories'))
        self.assertTrue(isinstance(the_election["categories"][0], unicode))
        self.assertTrue(the_election.has_key('background_categories'))
        self.assertTrue(isinstance(the_election["background_categories"][0], unicode))
        self.assertTrue(the_election.has_key('personal_data'))
        self.assertTrue(isinstance(the_election["personal_data"][0], unicode))
        

    #categories
    def test_get_categories_valid_json(self):
        response = self.api_client.get(
            '/api/v2/category/', 
            format='json', 
            authentication=self.get_credentials())
        self.assertValidJSONResponse(response)

    #@skip('aun no es testetado')
    def test_get_all_categories(self):
        response = self.api_client.get(
            '/api/v2/category/', 
            format='json', 
            authentication=self.get_credentials())
        categories = self.deserialize(response)['objects']
        
        self.assertEquals(len(categories),2)
        the_category = categories[0]
        self.assertEquals(the_category['id'],self.category1.id)
        self.assertEquals(the_category['name'],self.category1.name)
        the_category2 = categories[1]
        self.assertEquals(the_category2['id'],self.category2.id)
        self.assertEquals(the_category2['name'],self.category2.name)
        self.assertTrue(the_category.has_key("questions"))
        self.assertTrue(isinstance(the_category["questions"][0], unicode))

    def test_get_all_questions(self):
        response = self.api_client.get(
            '/api/v2/question/', 
            format='json', 
            authentication=self.get_credentials())

        questions = self.deserialize(response)['objects']

        self.assertEquals(len(questions),2)
        the_question = questions[0]
        self.assertTrue(the_question.has_key('question'))
        self.assertTrue(the_question.has_key('resource_uri'))
        self.assertEqual(the_question["resource_uri"], "/api/v2/question/{0}/".format(self.question_category_1.id))
        self.assertTrue(the_question.has_key('category'))
        self.assertTrue(isinstance(the_question["category"], unicode))
        self.assertEquals(the_question["category"], "/api/v2/category/{0}/".format(self.question_category_1.category.id))

    def test_get_all_answers(self):
        self.candidate.associate_answer(self.answer_for_question_1)

        response = self.api_client.get(
            '/api/v2/answer/', 
            format='json', 
            authentication=self.get_credentials())

        answers = self.deserialize(response)['objects']
        
        self.assertEquals(len(answers),Answer.objects.count())
        the_answer = answers[0]
        self.assertTrue(the_answer.has_key('caption'))
        self.assertTrue(the_answer.has_key('resource_uri'))
        self.assertEqual(the_answer["resource_uri"], "/api/v2/answer/{0}/".format(self.answer_for_question_1.id))
        self.assertTrue(the_answer.has_key('candidates'))
        self.assertEquals(len(the_answer["candidates"]), 1)
        self.assertTrue(isinstance(the_answer["candidates"][0], unicode))
        self.assertTrue(the_answer.has_key('question'))
        self.assertTrue(isinstance(the_answer["question"], unicode))
        self.assertEquals(the_answer["question"], "/api/v2/question/{0}/".format(self.answer_for_question_1.question.id))

    def test_get_personal_data_candidate(self):
        response = self.api_client.get(
            '/api/v2/personal_data_candidate/', 
            format='json', 
            authentication=self.get_credentials())

        self.assertHttpOK(response)
        personal_data_candidate = self.deserialize(response)['objects']
        # print personal_data_candidate[0]
        self.assertTrue(personal_data_candidate[0].has_key('candidate'))
        self.assertTrue(personal_data_candidate[0]['candidate'])
        self.assertTrue(personal_data_candidate[0].has_key('personal_data'))
        self.assertTrue(personal_data_candidate[0]['personal_data'])
        self.assertTrue(personal_data_candidate[0].has_key('value'))

    def test_get_personal_data(self):
        response = self.api_client.get(
            '/api/v2/personal_data/', 
            format='json', 
            authentication=self.get_credentials())

        self.assertHttpOK(response)

    def test_get_background(self):
        response = self.api_client.get(
            '/api/v2/background/', 
            format='json', 
            authentication=self.get_credentials())

        self.assertHttpOK(response)
        background = self.deserialize(response)['objects']
        self.assertIn('background_category', background[0])

    def test_get_first_background(self):
        response = self.api_client.get(
            '/api/v2/background/{0}/'.format(self.background_school.id), 
            format='json', 
            authentication=self.get_credentials())

        self.assertHttpOK(response)
        background = self.deserialize(response)
        expected_resource_uri = "/api/v2/background_category/{0}/".format(
            self.background_school.category.id
            )
        self.assertEquals(background["background_category"],expected_resource_uri)


    def test_get_backgrounds_candidate(self):
        response = self.api_client.get(
            '/api/v2/backgrounds_candidate/', 
            format='json', 
            authentication=self.get_credentials())

        self.assertHttpOK(response)
        backgrounds_candidate = self.deserialize(response)['objects']
        # print(backgrounds_candidate)
        self.assertTrue(backgrounds_candidate[0].has_key('value'))
        self.assertIn('background', backgrounds_candidate[0])

    def test_backgrounds_candidate_detail(self):
        response = self.api_client.get(
            '/api/v2/backgrounds_candidate/{0}/'.format(self.background_candidate.id), 
            format='json', 
            authentication=self.get_credentials())

        self.assertHttpOK(response)
        the_background_candidate = self.deserialize(response)
        self.assertEqual(the_background_candidate['value'], self.background_candidate.value)
        self.assertIn("background",the_background_candidate)


    # @skip('alguna razon')
    def test_get_all_candidates(self):
        response = self.api_client.get(
            '/api/v2/candidate/', 
            format='json', 
            authentication=self.get_credentials())

        self.assertHttpOK(response)
        candidates = self.deserialize(response)['objects']
        
        self.assertEquals(len(candidates),2)
        the_candidate = candidates[0]
        # print(the_candidate)
        self.assertTrue(the_candidate.has_key('slug'))
        self.assertFalse(the_candidate.has_key('personal_data'))
        self.assertTrue(the_candidate.has_key('personal_data_candidate'))
        self.assertTrue(isinstance(the_candidate["personal_data_candidate"][0], unicode))
        self.assertFalse(the_candidate.has_key('backgrounds'))
        self.assertTrue(the_candidate.has_key('backgrounds_candidate'))
        self.assertTrue(isinstance(the_candidate["backgrounds_candidate"][0], unicode))
        self.assertTrue(the_candidate.has_key('links'))
        self.assertTrue(isinstance(the_candidate["links"][0], unicode))
        self.assertTrue(the_candidate.has_key('answers'))
        self.assertTrue(isinstance(the_candidate["answers"][0], unicode))



    def test_get_background_category(self):
        response = self.api_client.get(
            '/api/v2/background_category/', 
            format='json', 
            authentication=self.get_credentials())

        self.assertHttpOK(response)
        background_category = self.deserialize(response)['objects']
        the_background_category = background_category[0]
        
        self.assertTrue(the_background_category.has_key('name'))
        self.assertTrue(the_background_category.has_key('background'))
        self.assertTrue(isinstance(the_background_category["background"][0], unicode))

    def test_get_links(self):
        response = self.api_client.get(
            '/api/v2/link/', 
            format='json', 
            authentication=self.get_credentials())

        self.assertHttpOK(response)
        link = self.deserialize(response)['objects']
        # print link
        the_link = link[0]
        self.assertTrue(the_link.has_key('name'))
        self.assertTrue(the_link.has_key('url'))

    def test_answer_filtering(self):
        filter_data = {
            'question': self.question_category_1.id
        }

        response = self.api_client.get(
            '/api/v2/answer/', 
            format='json', 
            authentication=self.get_credentials(),
            data=filter_data)
        
        self.assertHttpOK(response)
        answer = self.deserialize(response)['objects']
        self.assertEquals(len(answer), Answer.objects.filter(question=self.question_category_1).count())
        self.assertEquals( answer[0]['id'] , self.answer_for_question_1.id )

    # @skip('ignore')
    def test_answer_filtering_two_param(self):
        filter_data = {
            # 'question': self.question_category_1.id,
            'candidate__id': self.candidate.id
        }

        response = self.api_client.get(
            '/api/v2/answer/', 
            format='json', 
            authentication=self.get_credentials(),
            data=filter_data)
        # print response

        self.assertHttpOK(response)
        answer = self.deserialize(response)['objects']

        # self.assertEquals(len(answer), 1)
        self.assertEquals( answer[0]['id'] , self.answer_for_question_1.id )

    def test_filter_candidate(self):
        filter_data = {
            'id' : self.candidate.id
        }

        response = self.api_client.get(
            '/api/v2/candidate/', 
            format='json', 
            authentication=self.get_credentials(),
            data=filter_data)
        self.assertHttpOK(response)
        candidate = self.deserialize(response)['objects']
        self.assertEquals( len(candidate), 1)
        self.assertEquals( candidate[0]['id'], self.candidate.id )

    #@skip('')
    def test_media_naranja_post(self):
        answers = [self.answer_for_question_1.pk, self.answer_for_question_2.pk]
        questions_ids = [self.question_category_1.id, self.question_category_2.id]
        importances = [5, 3]
        importances_by_category = [5, 3]
        factor_question1 = (answers[0] == self.answer_for_question_1.pk) * importances[0]
        factor_question2 = (answers[1] == self.answer_for_question_2.pk) * importances[1]
        score_category1 = factor_question1 * 100.0 / importances_by_category[0]
        score_category2 = factor_question2 * 100.0 / importances_by_category[1]
        global_score = (factor_question1 + factor_question2) * 100.0 / sum(importances_by_category)
        response = self.api_client.post('/api/v2/medianaranja/', 
            format='json', 
            authentication=self.get_credentials(),
            data = {
                'data' : {
                    'question-0': answers[0], 'question-1': answers[1], \
                    'importance-0': importances[0], 'importance-1': importances[1],\
                    'question-id-0': questions_ids[0], 'question-id-1': questions_ids[1]
                    },\
                'election-id' : self.election.id
            }
        )
        expected_winner = {
                'global_score':global_score, 
                'category_score':[
                    {
                    'category':self.question_category_1.category.name,
                    'score': score_category1
                    },
                    {
                    'category':self.question_category_2.category.name,
                    'score': score_category2
                    },
                ],
                'candidate':self.candidate.id
                }
        expected_others = [
                {
                'global_score':0.0, 
                'category_score':[
                    {
                    'category':self.question_category_1.category.name,
                    'score': 0.0
                    },
                    {
                    'category':self.question_category_2.category.name,
                    'score': 0.0
                    },
                ],
                'candidate':self.candidate2.id
                }
        ]
        #de nuevo
        self.assertHttpCreated(response)
        self.assertValidJSON(response.content)

        candidates = self.deserialize(response)
        self.assertIn('winner', candidates)
        self.assertEquals(candidates['winner'],expected_winner)
        self.assertEquals(candidates['others'],expected_others)

    def test_media_naranja_post_jsonp(self):
        answers = [self.answer_for_question_1.pk, self.answer_for_question_2.pk]
        questions_ids = [self.question_category_1.id, self.question_category_2.id]
        importances = [5, 3]
        importances_by_category = [5, 3]
        factor_question1 = (answers[0] == self.answer_for_question_1.pk) * importances[0]
        factor_question2 = (answers[1] == self.answer_for_question_2.pk) * importances[1]
        score_category1 = factor_question1 * 100.0 / importances_by_category[0]
        score_category2 = factor_question2 * 100.0 / importances_by_category[1]
        global_score = (factor_question1 + factor_question2) * 100.0 / sum(importances_by_category)
        response = self.api_client.post('/api/v2/medianaranja/?callback=callback', 
            format='json', 
            authentication=self.get_credentials(),
            data = {
                
                'data' : {
                    'question-0': answers[0], 'question-1': answers[1], \
                    'importance-0': importances[0], 'importance-1': importances[1],\
                    'question-id-0': questions_ids[0], 'question-id-1': questions_ids[1]
                    },\
                'election-id' : self.election.id
            }
        )
        expected_winner = {
                'global_score':global_score, 
                'category_score':[
                    {
                    'category':self.question_category_1.category.name,
                    'score': score_category1
                    },
                    {
                    'category':self.question_category_2.category.name,
                    'score': score_category2
                    },
                ],
                'candidate':self.candidate.id
                }
        expected_others = [
                {
                'global_score':0.0, 
                'category_score':[
                    {
                    'category':self.question_category_1.category.name,
                    'score': 0.0
                    },
                    {
                    'category':self.question_category_2.category.name,
                    'score': 0.0
                    },
                ],
                'candidate':self.candidate2.id
                }
        ]

        self.assertTrue(response.content.startswith("callback("))
        content = response.content

        content = content.strip("callback(")

    def test_get_information_source(self):
        information_source = InformationSource.objects.create(
            candidate=self.candidate,
            question=self.question_category_1,
            content=u"Tomé esta información desde una página web")

        response = self.api_client.get(
            '/api/v2/information_source/', 
            format='json', 
            authentication=self.get_credentials())

        self.assertHttpOK(response)
        information_source_dict = self.deserialize(response)['objects'][0]
        self.assertEquals(information_source_dict['candidate'], '/api/v2/candidate/{0}/'.format(self.candidate.id))
        self.assertEquals(information_source_dict['question'], '/api/v2/question/{0}/'.format(self.question_category_1.id))
        self.assertEquals(information_source_dict['content'], information_source.content)


        response_question = self.api_client.get(
            information_source_dict['question'], 
            format='json', 
            authentication=self.get_credentials())


        self.assertHttpOK(response_question)
        question_dict = self.deserialize(response_question)

        self.assertIn('information_sources', question_dict)
        self.assertEquals(len(question_dict['information_sources']), 1)
        self.assertEquals(question_dict['information_sources'][0], '/api/v2/information_source/{0}/'.format(information_source.id))

        


from django.conf import settings
from django.test import TestCase
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.db import IntegrityError

from elections.models import Election, BackgroundCategory, Background, BackgroundCandidate, Candidate
from elections.forms import BackgroundForm, BackgroundCandidateForm


class BackgroundModelTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz')
        self.background_category, created = BackgroundCategory.objects.get_or_create(election=self.election,
                                                                    name='FooBar')

    def test_create_background(self):
        background, created = Background.objects.get_or_create(category=self.background_category,
                                                                name='foo')
        self.assertTrue(created)
        self.assertEqual(background.name, 'foo')
        self.assertEqual(background.category, self.background_category)


class BackgroundCreateView(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz')
        self.background_category, created = BackgroundCategory.objects.get_or_create(election=self.election,
                                                                    name='FooBar')

    def test_create_background_by_user_without_login(self):
        request = self.client.get(reverse('background_create',
                                    kwargs={'background_category_pk': self.background_category.pk}))
        self.assertEquals(request.status_code, 302)

    def test_create_background_by_user_success(self):
        self.client.login(username='joe', password='doe')
        request = self.client.get(reverse('background_create',
                                    kwargs={'background_category_pk': self.background_category.pk}))

        self.assertEqual(request.status_code, 200)
        self.assertTrue('form' in request.context)
        self.assertTrue(isinstance(request.context['form'], BackgroundForm))
        self.assertTrue('background_category' in request.context)
        self.assertTrue(isinstance(request.context['background_category'], BackgroundCategory))

    def test_post_background_create_without_login(self):
        params = {'name': 'Bar'}
        response = self.client.post(reverse('background_create',
                                        kwargs={'background_category_pk': self.background_category.pk}),
                                    params)

        self.assertEquals(response.status_code, 302)

    def test_get_background_create_with_login_stranger_background_category(self):
        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('background_create',
                                    kwargs={'background_category_pk': 97965678765}))
        self.assertEquals(response.status_code, 404)

    def test_post_background_create_with_login_stranger_background_category(self):
        self.client.login(username='joe', password='doe')

        params = {'name': 'Bar'}
        response = self.client.post(reverse('background_create',
                                        kwargs={'background_category_pk': 23678543567}),
                                    params)
        self.assertEquals(response.status_code, 404)

    def test_post_background_create_logged(self):
        self.client.login(username='joe', password='doe')

        params = {'name': 'Bar'}
        response = self.client.post(reverse('background_create',
                                        kwargs={'background_category_pk': self.background_category.pk}),
                                    params,
                                    follow=True)

        self.assertEquals(response.status_code, 200)
        qs = Background.objects.filter(name='Bar')
        self.assertEquals(qs.count(), 1)
        background = qs.get()
        self.assertEqual(background.name, params['name'])
        self.assertEqual(background.category, self.background_category)

        self.assertRedirects(response, reverse('background_category_create',
                                               kwargs={'election_slug': self.election.slug}))


class BackgroundCandidateModelTest(TestCase):
    def test_background_candidate_create(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz')
        self.background_category, created = BackgroundCategory.objects.get_or_create(election=self.election,
                                                                    name='FooBar')

        self.background, created = Background.objects.get_or_create(category=self.background_category,
                                                                name='foo')

        self.candidate, created = Candidate.objects.get_or_create(name='Juan Candidato',
                                                            election=self.election)

        background_candidate, created = BackgroundCandidate.objects.get_or_create(candidate=self.candidate,
                                                                                       background=self.background,
                                                                                       value='new_value')

        self.assertEqual(background_candidate.candidate, self.candidate)
        self.assertEqual(background_candidate.background, self.background)
        self.assertEqual(background_candidate.value, 'new_value')

class BackgroundCandidateCreateViewTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz')
        #deleting all background categories by default
        for backgroundcategory in self.election.backgroundcategory_set.all():
            backgroundcategory.delete()
            
        self.background_category, created = BackgroundCategory.objects.get_or_create(election=self.election,
                                                                    name='FooBar')

        self.background, created = Background.objects.get_or_create(category=self.background_category,
                                                                name='foo')

        self.candidate, created = Candidate.objects.get_or_create(name='Juan Candidato',
                                                            slug='juan-candidato',
                                                            election=self.election)

        self.user2 = User.objects.create_user(username='johnny', password='doe', email='johnny@doe.cl')

        self.election2, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user2,
                                                            slug='barbaz')
        self.background_category2, created = BackgroundCategory.objects.get_or_create(election=self.election2,
                                                                    name='FooBar')

        self.background2, created = Background.objects.get_or_create(category=self.background_category2,
                                                                name='foo')

        self.candidate2, created = Candidate.objects.get_or_create(name='Juan Candidato',
                                                            slug='juan-candidato',
                                                            election=self.election2)

    def test_create_background_candidate_by_user_without_login(self):
        response = self.client.get(reverse('background_candidate_create',
                                    kwargs={'candidate_pk': self.candidate.pk,
                                            'background_pk': self.background.pk}))
        self.assertEquals(response.status_code, 302)

    def test_create_background_candidate_by_user_success(self):
        self.client.login(username='joe', password='doe')
        request = self.client.get(reverse('background_candidate_create',
                                    kwargs={'candidate_pk': self.candidate.pk,
                                            'background_pk': self.background.pk}))

        self.assertEqual(request.status_code, 200)
        self.assertTrue('form' in request.context)
        self.assertTrue(isinstance(request.context['form'], BackgroundCandidateForm))
        self.assertTrue('candidate' in request.context)
        self.assertTrue(isinstance(request.context['candidate'], Candidate))
        self.assertTrue('background' in request.context)
        self.assertTrue(isinstance(request.context['background'], Background))

    def test_post_background_candidate_create_without_login(self):
        params = {'value': 'Bar'}
        response = self.client.get(reverse('background_candidate_create',
                                    kwargs={'candidate_pk': self.candidate.pk,
                                            'background_pk': self.background.pk}),
                                    params)

        self.assertEquals(response.status_code, 302)

    def test_get_background_candidate_create_with_login_stranger_background_category(self):
        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('background_candidate_create',
                                    kwargs={'candidate_pk': self.candidate2.pk,
                                            'background_pk': self.background2.pk}))
        self.assertEquals(response.status_code, 404)

    def test_post_background_candidate_create_with_login_stranger_background_category(self):
        self.client.login(username='joe', password='doe')

        params = {'value': 'Bar'}
        response = self.client.get(reverse('background_candidate_create',
                                    kwargs={'candidate_pk': self.candidate2.pk,
                                            'background_pk': self.background2.pk}),
                                    params)
        self.assertEquals(response.status_code, 404)

    def test_post_background_create_logged(self):
        self.client.login(username='joe', password='doe')

        params = {'value': 'Bar'}
        response = self.client.post(reverse('background_candidate_create',
                                    kwargs={'candidate_pk': self.candidate.pk,
                                            'background_pk': self.background.pk}),
                                    params,
                                    follow=True)

        self.assertEquals(response.status_code, 200)
        self.assertEquals(response.content, '{"value": "%s"}' % params['value'])
        
        expected_dict = {
        1: {'name': self.background_category.name,
            'backgrounds': {
                1: {'name': self.background.name, 'value': 'Bar'},
                }
            },
        }
        self.assertEquals(self.candidate.get_background, expected_dict)


class AsyncCreateBackgroundViewTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz')
        self.background_category, created = BackgroundCategory.objects.get_or_create(election=self.election,
                                                                    name='FooBar')

        self.background, created = Background.objects.get_or_create(category=self.background_category,
                                                                name='foo')

        self.user2 = User.objects.create_user(username='johnny', password='doe', email='johnny@doe.cl')

        self.election2, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user2,
                                                            slug='barbaz')
        self.background_category2, created = BackgroundCategory.objects.get_or_create(election=self.election2,
                                                                    name='FooBar')

        self.background2, created = Background.objects.get_or_create(category=self.background_category2,
                                                                name='foo')

    def test_get_async_create_background_with_login(self):
        self.client.login(username='joe', password='doe')
        response = self.client.get(reverse('async_create_background',
                                    kwargs={'background_category_pk': self.background_category.pk}))
        self.assertEqual(response.status_code, 405)

    def test_get_async_create_background_without_login(self):
        response = self.client.get(reverse('async_create_background',
                                    kwargs={'background_category_pk': self.background_category.pk}))
        self.assertEqual(response.status_code, 302)


    def test_post_async_create_background_without_login(self):
        params = {'value': 'Bar'}
        response = self.client.post(reverse('async_create_background',
                                    kwargs={'background_category_pk': self.background_category.pk}),
                                    params)

        self.assertEquals(response.status_code, 302)

    def test_post_async_create_background_with_login_stranger_background_category(self):
        self.client.login(username='joe', password='doe')

        params = {'value': 'Bar'}
        response = self.client.post(reverse('async_create_background',
                                    kwargs={'background_category_pk': self.background_category2.pk}),
                                    params)
        self.assertEquals(response.status_code, 404)

    def test_post_async_create_background_logged(self):
        self.client.login(username='joe', password='doe')

        params = {'value': 'Bar'}
        response = self.client.post(reverse('async_create_background',
                                    kwargs={'background_category_pk': self.background_category.pk}),
                                    params,
                                    follow=True)

        self.assertEquals(response.status_code, 200)

        backgrounds = self.background_category.background_set.all()
        background_names = [ background.name for background in backgrounds]

        self.assertTrue(params['value'] in background_names)
        background = self.background_category.background_set.get(name=params['value'])
        self.assertEquals(response.content, '{"pk": %d, "name": "%s"}' % (background.pk, params['value']))


class AsyncDeleteBackgroundTest(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(username='joe', password='doe', email='joe@doe.cl')
        self.election, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=self.user,
                                                            slug='barbaz')
        self.background_category, created = BackgroundCategory.objects.get_or_create(election=self.election,
                                                                    name='FooBar')

        self.background, created = Background.objects.get_or_create(category=self.background_category,
                                                                name='foo')


    def test_post_with_login(self):
        self.client.login(username='joe', password='doe')

        request = self.client.post(reverse('async_delete_background',
                                kwargs={'background_pk': self.background.pk}),
                                        {})
        self.assertEquals(request.status_code, 200)

    def test_post_without_login(self):
        request = self.client.post(reverse('async_delete_background',
                                kwargs={'background_pk': self.background.pk}),
                                        {})
        self.assertEquals(request.status_code, 302)


    def test_get_405(self):
        self.client.login(username='joe', password='doe')
        request = self.client.get(reverse('async_delete_background',
                                kwargs={'background_pk': self.background.pk}))

        self.assertEquals(request.status_code, 405)

    def test_post_with_stranger_background(self):
        user2 = User.objects.create_user(username='johnny', password='doe', email='johnny@doe.cl')

        election2, created = Election.objects.get_or_create(name='BarBaz',
                                                            owner=user2,
                                                            slug='barbaz')
        background_category2, created = BackgroundCategory.objects.get_or_create(election=election2,
                                                                    name='FooBar')

        background2, created = Background.objects.get_or_create(category=background_category2,
                                                                name='foo')

        self.client.login(username='joe', password='doe')
        request = self.client.post(reverse('async_delete_background',
                                kwargs={'background_pk': background2.pk}))

        self.assertEquals(request.status_code, 404)

    def test_post_success(self):
        self.client.login(username='joe', password='doe')
        temp_pk = self.background.pk
        request = self.client.post(reverse('async_delete_background',
                                kwargs={'background_pk': self.background.pk}),
                                        {})

        self.assertEquals(request.status_code, 200)
        self.assertEquals(request.content, '{"result": "OK"}')

        self.assertRaises(Background.DoesNotExist, Background.objects.get, pk=temp_pk)



from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.forms import formsets
from django.http import HttpResponse, Http404
from django.shortcuts import render_to_response, redirect, get_object_or_404
from django.template import RequestContext
from django.template.context import RequestContext
from django.utils import simplejson as json
from django.utils.decorators import method_decorator
from django.views.decorators.http import require_http_methods
from django.views.generic import CreateView, DetailView, UpdateView
from django.views.decorators.csrf import csrf_exempt

from elections.models import Election, Candidate, Answer, Category, Question, Visitor, VisitorAnswer, VisitorScore, CategoryScore

# MediaNaranja Views
@login_required
@require_http_methods(['GET', 'POST'])
def associate_answer_to_candidate(request, candidate_slug, election_slug):
    election = get_object_or_404(Election, slug=election_slug, owner=request.user)
    candidate = get_object_or_404(Candidate, slug=candidate_slug, election=election)
    if request.POST:
        answer_id = request.POST.get('answer', None)
        answer = get_object_or_404(Answer, pk=answer_id, question__category__election=election)
        candidate.associate_answer(answer)
        return HttpResponse(json.dumps({'answer': answer.pk}),
                            content_type='application/json')
    return render_to_response(\
            'elections/associate_answer.html', {'candidate': candidate, 'categories': election.category_set},
            context_instance=RequestContext(request))


def strip_elements_from_dictionary(dictionary, election):
    candidates = election.candidate_set.all()
    categories = election.category_set.all()

    number_of_questions = 0
    for c in categories:
        number_of_questions += len(c.question_set.all())

    importances = []
    answers = []
    questions = []

    for i in range(number_of_questions):
        if 'question-'+str(i) in dictionary:
            ans_id = int(dictionary['question-'+str(i)])
        else:
            ans_id = -1
        question_id = int(dictionary['question-id-'+str(i)])
        answers.append(Answer.objects.filter(id=ans_id))
        importances.append(int(dictionary['importance-'+str(i)]))
        questions.append(Question.objects.get(id=question_id))

    return {
        'answers': answers, 
        'importances' : importances, 
        'questions' : questions, 
        'candidates' : candidates, 
        'categories' : categories
    }

def post_medianaranja1(request, username, election_slug):
    user_list = User.objects.filter(username=username)
    if user_list.count() == 0:
        raise Http404

    election = Election.objects.get(slug=election_slug, owner=User.objects.get(username=user_list[0]))

    elements = strip_elements_from_dictionary(request.POST, election)

    return medianaranja2(elements["answers"], elements['importances'], elements['questions'], elements['candidates'], elements['categories'], election)

def get_medianaranja1(request, username, election_slug):
    election = get_object_or_404(Election, owner__username=username, slug=election_slug)

    send_to_template = []
    counter = 0
    check = False
    for x in election.category_set.all():
        empty_questions = []
        list_questions = x.question_set.all()
        for i in range(len(list_questions)):
            y = list_questions[i]
            empty_questions.append((counter,y,y.answer_set.all()))
            counter += 1
        send_to_template.append((x,empty_questions))
        for tupla in send_to_template:
            if len(tupla[1]) > 0:
                check = True
                break

    return {'stt':send_to_template, 'check': check, 'election': election}

def medianaranja1(request, username, election_slug):

    if request.method == "POST":
        context = post_medianaranja1(request, username, election_slug)
        return render_to_response('medianaranja2.html', context, context_instance = RequestContext(request))

    else:
        context = get_medianaranja1(request, username, election_slug)
        return render_to_response('medianaranja1.html', context, context_instance = RequestContext(request))

@csrf_exempt
def medianaranja1_embed(request, username, election_slug):
    if request.method == "POST":
        context = post_medianaranja1(request, username, election_slug)
        return render_to_response('elections/embeded/medianaranja2.html', context, context_instance = RequestContext(request))
    else:
        context = get_medianaranja1(request, username, election_slug)
        return render_to_response('elections/embeded/medianaranja1.html', context, context_instance = RequestContext(request))

def medianaranja2(my_answers, importances, questions, candidates, categories, election):
    election_url=reverse("election_detail",kwargs={'username': election.owner.username, 'slug':election.slug})
    visitor = Visitor(election=election, election_url=election_url)
    visitor.save()
    #save answers for latter analysis:
    for i, importance in enumerate(importances):
        if my_answers[i]:
            visitoranswer = VisitorAnswer(visitor=visitor,answer=my_answers[i][0], answer_importance=importance)
        else:
            visitoranswer = VisitorAnswer(visitor=visitor,answer_text="",question_text=questions[i].question,\
                question_category_text=questions[i].category.name, answer_importance=importance)
        visitoranswer.save()

    scores_and_candidates = []
    for candidate in candidates:
        score = candidate.get_score(my_answers, importances)
        global_score = score[0]
        category_scores = score[1]
        visitor_score = VisitorScore.objects.create(visitor=visitor, candidate_name=candidate.name,score=global_score)
        for i,category_score in enumerate(category_scores):
            categoryscore = CategoryScore.objects.create(visitor_score=visitor_score,category_score=category_score, category_name=categories[i])

        scores_and_candidates.append([global_score,category_scores,candidate])
    scores_and_candidates.sort()
    scores_and_candidates.reverse()

    winner = scores_and_candidates[0]
    other_candidates = scores_and_candidates[1:]
    


    

    context = {'election':election, 'categories':categories,'winner':winner,'others':other_candidates}
    return context
